
from collections import OrderedDict
from pprint import pformat
from rubikscubennnsolver.RubiksCubeNNNOddEdges import RubiksCubeNNNOddEdges
from rubikscubennnsolver.RubiksCube555 import RubiksCube555, solved_5x5x5
from rubikscubennnsolver.RubiksCube666 import RubiksCube666, solved_6x6x6, moves_6x6x6
from rubikscubennnsolver.RubiksSide import Side, SolveError
from rubikscubennnsolver.LookupTable import LookupTable, LookupTableIDA
import logging
import math
import os
import random
import re
import subprocess
import sys

log = logging.getLogger(__name__)

moves_7x7x7 = moves_6x6x6
solved_7x7x7 = 'UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUURRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB'


class LookupTable777UDObliqueEdgePairingMiddleOnly(LookupTable):
    """
    24!/(8!*16!) is 735,471

    lookup-table-7x7x7-step11-UD-oblique-edge-pairing-middle-only.txt
    =================================================================
    1 steps has 5 entries (0 percent, 0.00x previous step)
    2 steps has 66 entries (0 percent, 13.20x previous step)
    3 steps has 850 entries (0 percent, 12.88x previous step)
    4 steps has 8,753 entries (1 percent, 10.30x previous step)
    5 steps has 67,758 entries (9 percent, 7.74x previous step)
    6 steps has 243,959 entries (33 percent, 3.60x previous step)
    7 steps has 257,410 entries (34 percent, 1.06x previous step)
    8 steps has 135,684 entries (18 percent, 0.53x previous step)
    9 steps has 20,177 entries (2 percent, 0.15x previous step)
    10 steps has 801 entries (0 percent, 0.04x previous step)
    11 steps has 8 entries (0 percent, 0.01x previous step)

    Total: 735,471 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step11-UD-oblique-edge-pairing-middle-only.txt',
            '08088080000000000000000000000000404404',
            linecount=735471)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Upper
            'x', 'x', parent_state[11], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[23], 'x', 'x', 'x', parent_state[27],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[39], 'x', 'x',

            # Left
            'x', 'x', parent_state[60], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[72], 'x', 'x', 'x', parent_state[76],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[88], 'x', 'x',

            # Front
            'x', 'x', parent_state[109], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[121], 'x', 'x', 'x', parent_state[125],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[137], 'x', 'x',

            # Right
            'x', 'x', parent_state[158], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[170], 'x', 'x', 'x', parent_state[174],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[186], 'x', 'x',

            # Back
            'x', 'x', parent_state[207], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[219], 'x', 'x', 'x', parent_state[223],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[235], 'x', 'x',

            # Down
            'x', 'x', parent_state[256], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[268], 'x', 'x', 'x', parent_state[272],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[284], 'x', 'x'
        ]

        result = ['1' if x in ('U', 'D') else '0' for x in result]
        result = ''.join(result)

        # Convert to hex
        return self.hex_format % int(result, 2)


class LookupTable777UDObliqueEdgePairingLeftOnly(LookupTable):
    """
    24!/(8!*16!) is 735,471

    lookup-table-7x7x7-step12-UD-oblique-edge-pairing-left-only.txt
    ===============================================================
    1 steps has 5 entries (0 percent, 0.00x previous step)
    2 steps has 66 entries (0 percent, 13.20x previous step)
    3 steps has 850 entries (0 percent, 12.88x previous step)
    4 steps has 8,753 entries (1 percent, 10.30x previous step)
    5 steps has 67,758 entries (9 percent, 7.74x previous step)
    6 steps has 243,959 entries (33 percent, 3.60x previous step)
    7 steps has 257,410 entries (34 percent, 1.06x previous step)
    8 steps has 135,684 entries (18 percent, 0.53x previous step)
    9 steps has 20,177 entries (2 percent, 0.15x previous step)
    10 steps has 801 entries (0 percent, 0.04x previous step)
    11 steps has 8 entries (0 percent, 0.01x previous step)

    Total: 735,471 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step12-UD-oblique-edge-pairing-left-only.txt',
            '10104040000000000000000000000000808202',
            linecount=735471)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Upper
            'x', parent_state[10], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[20],
            'x', 'x', 'x', 'x', 'x',
            parent_state[30], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[40], 'x',

            # Left
            'x', parent_state[59], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[69],
            'x', 'x', 'x', 'x', 'x',
            parent_state[79], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[89], 'x',

            # Front
            'x', parent_state[108], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[118],
            'x', 'x', 'x', 'x', 'x',
            parent_state[128], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[138], 'x',

            # Right
            'x', parent_state[157], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[167],
            'x', 'x', 'x', 'x', 'x',
            parent_state[177], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[187], 'x',

            # Back
            'x', parent_state[206], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[216],
            'x', 'x', 'x', 'x', 'x',
            parent_state[226], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[236], 'x',

            # Down
            'x', parent_state[255], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[265],
            'x', 'x', 'x', 'x', 'x',
            parent_state[275], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[285], 'x'
        ]

        result = ['1' if x in ('U', 'D') else '0' for x in result]
        result = ''.join(result)

        # Convert to hex
        return self.hex_format % int(result, 2)


class LookupTable777UDObliqueEdgePairingRightOnly(LookupTable):
    """
    24!/(8!*16!) is 735,471

    lookup-table-7x7x7-step13-UD-oblique-edge-pairing-right-only.txt
    =================================================================
    1 steps has 5 entries (0 percent, 0.00x previous step)
    2 steps has 66 entries (0 percent, 13.20x previous step)
    3 steps has 850 entries (0 percent, 12.88x previous step)
    4 steps has 8,753 entries (1 percent, 10.30x previous step)
    5 steps has 67,758 entries (9 percent, 7.74x previous step)
    6 steps has 243,959 entries (33 percent, 3.60x previous step)
    7 steps has 257,410 entries (34 percent, 1.06x previous step)
    8 steps has 135,684 entries (18 percent, 0.53x previous step)
    9 steps has 20,177 entries (2 percent, 0.15x previous step)
    10 steps has 801 entries (0 percent, 0.04x previous step)
    11 steps has 8 entries (0 percent, 0.01x previous step)

    Total: 735,471 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step13-UD-oblique-edge-pairing-right-only.txt',
            '05000500000000000000000000000000280028',
            linecount=735471)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Upper
            'x', 'x', 'x', parent_state[12], 'x',
            parent_state[16], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[34],
            'x', parent_state[38], 'x', 'x', 'x',

            # Left
            'x', 'x', 'x', parent_state[61], 'x',
            parent_state[65], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[83],
            'x', parent_state[87], 'x', 'x', 'x',

            # Front
            'x', 'x', 'x', parent_state[110], 'x',
            parent_state[114], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[132],
            'x', parent_state[136], 'x', 'x', 'x',

            # Right
            'x', 'x', 'x', parent_state[159], 'x',
            parent_state[163], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[181],
            'x', parent_state[185], 'x', 'x', 'x',

            # Back
            'x', 'x', 'x', parent_state[208], 'x',
            parent_state[212], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[230],
            'x', parent_state[234], 'x', 'x', 'x',

            # Down
            'x', 'x', 'x', parent_state[257], 'x',
            parent_state[261], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[279],
            'x', parent_state[283], 'x', 'x', 'x'
        ]

        result = ['1' if x in ('U', 'D') else '0' for x in result]
        result = ''.join(result)

        # Convert to hex
        return self.hex_format % int(result, 2)


class LookupTableIDA777UDObliqueEdgePairing(LookupTableIDA):
    """
    lookup-table-7x7x7-step10-UD-oblique-edge-pairing.txt
    =====================================================
    1 steps has 5 entries (0 percent, 0.00x previous step)
    2 steps has 66 entries (0 percent, 13.20x previous step)
    3 steps has 916 entries (0 percent, 13.88x previous step)
    4 steps has 10,132 entries (0 percent, 11.06x previous step)
    5 steps has 92,070 entries (0 percent, 9.09x previous step)
    6 steps has 558,938 entries (0 percent, 6.07x previous step)
    7 steps has 4,163,342 entries (4 percent, 7.45x previous step)
    8 steps has 53,921,753 entries (56 percent, 12.95x previous step)
    9 steps has 33,108,789 entries (34 percent, 0.61x previous step)
    10 steps has 3,448,937 entries (3 percent, 0.10x previous step)
    11 steps has 49,979 entries (0 percent, 0.01x previous step)

    Total: 95,354,927 entries
    """

    def __init__(self, parent):
        LookupTableIDA.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step10-UD-oblique-edge-pairing.txt',
            '1d18c5c0000000000000000000000000e8c62e',
            moves_7x7x7,

            # do not mess up UD 5x5x5 centers
            ("3Rw", "3Rw'", "3Lw", "3Lw'", "3Fw", "3Fw'", "3Bw", "3Bw'"),

            # prune tables
            (parent.lt_UD_oblique_edge_pairing_middle_only,
             parent.lt_UD_oblique_edge_pairing_left_only,
             parent.lt_UD_oblique_edge_pairing_right_only),
            linecount=5961573)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Upper
            'x', parent_state[10], parent_state[11], parent_state[12], 'x',
            parent_state[16], 'x', 'x', 'x', parent_state[20],
            parent_state[23], 'x', 'x', 'x', parent_state[27],
            parent_state[30], 'x', 'x', 'x', parent_state[34],
            'x', parent_state[38], parent_state[39], parent_state[40], 'x',

            # Left
            'x', parent_state[59], parent_state[60], parent_state[61], 'x',
            parent_state[65], 'x', 'x', 'x', parent_state[69],
            parent_state[72], 'x', 'x', 'x', parent_state[76],
            parent_state[79], 'x', 'x', 'x', parent_state[83],
            'x', parent_state[87], parent_state[88], parent_state[89], 'x',

            # Front
            'x', parent_state[108], parent_state[109], parent_state[110], 'x',
            parent_state[114], 'x', 'x', 'x', parent_state[118],
            parent_state[121], 'x', 'x', 'x', parent_state[125],
            parent_state[128], 'x', 'x', 'x', parent_state[132],
            'x', parent_state[136], parent_state[137], parent_state[138], 'x',

            # Right
            'x', parent_state[157], parent_state[158], parent_state[159], 'x',
            parent_state[163], 'x', 'x', 'x', parent_state[167],
            parent_state[170], 'x', 'x', 'x', parent_state[174],
            parent_state[177], 'x', 'x', 'x', parent_state[181],
            'x', parent_state[185], parent_state[186], parent_state[187], 'x',

            # Back
            'x', parent_state[206], parent_state[207], parent_state[208], 'x',
            parent_state[212], 'x', 'x', 'x', parent_state[216],
            parent_state[219], 'x', 'x', 'x', parent_state[223],
            parent_state[226], 'x', 'x', 'x', parent_state[230],
            'x', parent_state[234], parent_state[235], parent_state[236], 'x',

            # Down
            'x', parent_state[255], parent_state[256], parent_state[257], 'x',
            parent_state[261], 'x', 'x', 'x', parent_state[265],
            parent_state[268], 'x', 'x', 'x', parent_state[272],
            parent_state[275], 'x', 'x', 'x', parent_state[279],
            'x', parent_state[283], parent_state[284], parent_state[285], 'x'
        ]

        result = ['1' if x in ('U', 'D') else '0' for x in result]
        result = ''.join(result)

        # Convert to hex
        return self.hex_format % int(result, 2)


class LookupTable777LRObliqueEdgePairingMiddleOnly(LookupTable):
    """
    lookup-table-7x7x7-step21-LR-oblique-edge-pairing-middle-only.txt
    =================================================================
    1 steps has 2 entries (0 percent, 0.00x previous step)
    2 steps has 26 entries (0 percent, 13.00x previous step)
    3 steps has 190 entries (1 percent, 7.31x previous step)
    4 steps has 612 entries (4 percent, 3.22x previous step)
    5 steps has 1,513 entries (11 percent, 2.47x previous step)
    6 steps has 3,370 entries (26 percent, 2.23x previous step)
    7 steps has 4,066 entries (31 percent, 1.21x previous step)
    8 steps has 2,258 entries (17 percent, 0.56x previous step)
    9 steps has 803 entries (6 percent, 0.36x previous step)
    10 steps has 30 entries (0 percent, 0.04x previous step)

    Total: 12,870 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step21-LR-oblique-edge-pairing-middle-only.txt',
            '2022020000000808808000000',
            linecount=12870)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Left
            'x', 'x', parent_state[60], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[72], 'x', 'x', 'x', parent_state[76],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[88], 'x', 'x',

            # Front
            'x', 'x', parent_state[109], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[121], 'x', 'x', 'x', parent_state[125],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[137], 'x', 'x',

            # Right
            'x', 'x', parent_state[158], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[170], 'x', 'x', 'x', parent_state[174],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[186], 'x', 'x',

            # Back
            'x', 'x', parent_state[207], 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            parent_state[219], 'x', 'x', 'x', parent_state[223],
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', parent_state[235], 'x', 'x'
        ]

        result = ['1' if x in ('L', 'R') else '0' for x in result]
        result = ''.join(result)

        # Convert to hex
        return self.hex_format % int(result, 2)


class LookupTable777LRObliqueEdgePairingLeftOnly(LookupTable):
    """
    lookup-table-7x7x7-step22-LR-oblique-edge-pairing-left-only.txt
    ===============================================================
    1 steps has 2 entries (0 percent, 0.00x previous step)
    2 steps has 26 entries (0 percent, 13.00x previous step)
    3 steps has 210 entries (1 percent, 8.08x previous step)
    4 steps has 722 entries (5 percent, 3.44x previous step)
    5 steps has 1,752 entries (13 percent, 2.43x previous step)
    6 steps has 4,033 entries (31 percent, 2.30x previous step)
    7 steps has 4,014 entries (31 percent, 1.00x previous step)
    8 steps has 1,977 entries (15 percent, 0.49x previous step)
    9 steps has 134 entries (1 percent, 0.07x previous step)

    Total: 12,870 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step22-LR-oblique-edge-pairing-left-only.txt',
            '4041010000001010404000000',
            linecount=12870)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Left
            'x', parent_state[59], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[69],
            'x', 'x', 'x', 'x', 'x',
            parent_state[79], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[89], 'x',

            # Front
            'x', parent_state[108], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[118],
            'x', 'x', 'x', 'x', 'x',
            parent_state[128], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[138], 'x',

            # Right
            'x', parent_state[157], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[167],
            'x', 'x', 'x', 'x', 'x',
            parent_state[177], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[187], 'x',

            # Back
            'x', parent_state[206], 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[216],
            'x', 'x', 'x', 'x', 'x',
            parent_state[226], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', parent_state[236], 'x'
        ]

        result = ['1' if x in ('L', 'R') else '0' for x in result]
        result = ''.join(result)

        # Convert to hex
        return self.hex_format % int(result, 2)


class LookupTable777LRObliqueEdgePairingRightOnly(LookupTable):
    """
    lookup-table-7x7x7-step23-LR-oblique-edge-pairing-right-only.txt
    ================================================================
    1 steps has 2 entries (0 percent, 0.00x previous step)
    2 steps has 26 entries (0 percent, 13.00x previous step)
    3 steps has 210 entries (1 percent, 8.08x previous step)
    4 steps has 722 entries (5 percent, 3.44x previous step)
    5 steps has 1,752 entries (13 percent, 2.43x previous step)
    6 steps has 4,033 entries (31 percent, 2.30x previous step)
    7 steps has 4,014 entries (31 percent, 1.00x previous step)
    8 steps has 1,977 entries (15 percent, 0.49x previous step)
    9 steps has 134 entries (1 percent, 0.07x previous step)

    Total: 12,870 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step23-LR-oblique-edge-pairing-right-only.txt',
            '1400140000000500050000000',
            linecount=12870)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Left
            'x', 'x', 'x', parent_state[61], 'x',
            parent_state[65], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[83],
            'x', parent_state[87], 'x', 'x', 'x',

            # Front
            'x', 'x', 'x', parent_state[110], 'x',
            parent_state[114], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[132],
            'x', parent_state[136], 'x', 'x', 'x',

            # Right
            'x', 'x', 'x', parent_state[159], 'x',
            parent_state[163], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[181],
            'x', parent_state[185], 'x', 'x', 'x',

            # Back
            'x', 'x', 'x', parent_state[208], 'x',
            parent_state[212], 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', 'x',
            'x', 'x', 'x', 'x', parent_state[230],
            'x', parent_state[234], 'x', 'x', 'x'
        ]

        result = ['1' if x in ('L', 'R') else '0' for x in result]
        result = ''.join(result)

        # Convert to hex
        return self.hex_format % int(result, 2)


class LookupTableIDA777LRObliqueEdgePairing(LookupTableIDA):
    """
    lookup-table-7x7x7-step20-LR-oblique-edge-pairing.txt
    =====================================================
    1 steps has 2 entries (0 percent, 0.00x previous step)
    2 steps has 26 entries (0 percent, 13.00x previous step)
    3 steps has 214 entries (0 percent, 8.23x previous step)
    4 steps has 806 entries (0 percent, 3.77x previous step)
    5 steps has 3,006 entries (0 percent, 3.73x previous step)
    6 steps has 15,990 entries (0 percent, 5.32x previous step)
    7 steps has 87,030 entries (1 percent, 5.44x previous step)
    8 steps has 455,114 entries (9 percent, 5.23x previous step)
    9 steps has 1,784,216 entries (35 percent, 3.92x previous step)
    10 steps has 2,208,423 entries (43 percent, 1.24x previous step)
    11 steps has 469,086 entries (9 percent, 0.21x previous step)
    12 steps has 20,267 entries (0 percent, 0.04x previous step)
    13 steps has 100 entries (0 percent, 0.00x previous step)

    Total: 5,044,280 entries
    """

    def __init__(self, parent):
        LookupTableIDA.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step20-LR-oblique-edge-pairing.txt',
            '7463170000001d18c5c000000',
            moves_7x7x7,

            ("3Rw", "3Rw'", "3Lw", "3Lw'", "3Fw", "3Fw'", "3Bw", "3Bw'", # do not mess up UD 5x5x5 centers
             "Rw",  "Rw'",  "Lw",  "Lw'",  "Fw",  "Fw'",  "Bw",  "Bw'", # do not mess up UD oblique edges
             "3Uw", "3Uw'", "3Dw", "3Dw'"),

            # prune tables
            (parent.lt_LR_oblique_edge_pairing_middle_only,
             parent.lt_LR_oblique_edge_pairing_left_only,
             parent.lt_LR_oblique_edge_pairing_right_only),
            linecount=5044280)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Left
            'x', parent_state[59], parent_state[60], parent_state[61], 'x',
            parent_state[65], 'x', 'x', 'x', parent_state[69],
            parent_state[72], 'x', 'x', 'x', parent_state[76],
            parent_state[79], 'x', 'x', 'x', parent_state[83],
            'x', parent_state[87], parent_state[88], parent_state[89], 'x',

            # Front
            'x', parent_state[108], parent_state[109], parent_state[110], 'x',
            parent_state[114], 'x', 'x', 'x', parent_state[118],
            parent_state[121], 'x', 'x', 'x', parent_state[125],
            parent_state[128], 'x', 'x', 'x', parent_state[132],
            'x', parent_state[136], parent_state[137], parent_state[138], 'x',

            # Right
            'x', parent_state[157], parent_state[158], parent_state[159], 'x',
            parent_state[163], 'x', 'x', 'x', parent_state[167],
            parent_state[170], 'x', 'x', 'x', parent_state[174],
            parent_state[177], 'x', 'x', 'x', parent_state[181],
            'x', parent_state[185], parent_state[186], parent_state[187], 'x',

            # Back
            'x', parent_state[206], parent_state[207], parent_state[208], 'x',
            parent_state[212], 'x', 'x', 'x', parent_state[216],
            parent_state[219], 'x', 'x', 'x', parent_state[223],
            parent_state[226], 'x', 'x', 'x', parent_state[230],
            'x', parent_state[234], parent_state[235], parent_state[236], 'x'
        ]

        result = ['1' if x in ('L', 'R') else '0' for x in result]
        result = ''.join(result)

        # Convert to hex
        return self.hex_format % int(result, 2)


class LookupTable777UDSolveInnerCentersAndObliqueEdgesCenterOnly(LookupTable):
    """
    lookup-table-7x7x7-step51-UD-solve-inner-center-and-oblique-edges-center-only.txt
    =================================================================================
    1 steps has 5 entries (0 percent, 0.00x previous step)
    2 steps has 22 entries (0 percent, 4.40x previous step)
    3 steps has 82 entries (1 percent, 3.73x previous step)
    4 steps has 292 entries (5 percent, 3.56x previous step)
    5 steps has 986 entries (20 percent, 3.38x previous step)
    6 steps has 2,001 entries (40 percent, 2.03x previous step)
    7 steps has 1,312 entries (26 percent, 0.66x previous step)
    8 steps has 200 entries (4 percent, 0.15x previous step)

    Total: 4,900 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step51-UD-solve-inner-center-and-oblique-edges-center-only.txt',
            'xxxxxxUUUxxUxUxxUUUxxxxxxxxxxxxDDDxxDxDxxDDDxxxxxx',
            linecount=4900)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Upper
            'xxxxx',
            'x', parent_state[17], parent_state[18], parent_state[19], 'x',
            'x', parent_state[24], 'x', parent_state[26], 'x',
            'x', parent_state[31], parent_state[32], parent_state[33], 'x',
            'xxxxx',

            # Down
            'xxxxx',
            'x', parent_state[262], parent_state[263], parent_state[264], 'x',
            'x', parent_state[269], 'x', parent_state[271], 'x',
            'x', parent_state[276], parent_state[277], parent_state[278], 'x',
            'xxxxx'
        ]

        result = ''.join(result)
        return result


class LookupTable777UDSolveInnerCentersAndObliqueEdgesEdgesOnly(LookupTable):
    """
    lookup-table-7x7x7-step52-UD-solve-inner-center-and-oblique-edges-edges-only.txt
    ================================================================================
    1 steps has 294 entries (0 percent, 0.00x previous step)
    2 steps has 1,392 entries (0 percent, 4.73x previous step)
    3 steps has 6,112 entries (1 percent, 4.39x previous step)
    4 steps has 23,304 entries (6 percent, 3.81x previous step)
    5 steps has 71,086 entries (20 percent, 3.05x previous step)
    6 steps has 137,528 entries (40 percent, 1.93x previous step)
    7 steps has 92,192 entries (26 percent, 0.67x previous step)
    8 steps has 11,068 entries (3 percent, 0.12x previous step)
    9 steps has 24 entries (0 percent, 0.00x previous step)

    Total: 343,000 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step52-UD-solve-inner-center-and-oblique-edges-edges-only.txt',
            ('xUUUxUxxxUUxxxUUxxxUxUUUxxDDDxDxxxDDxxxDDxxxDxDDDx',
             'xUUUxUxxxUUxxxUUxxxUxDDDxxUUUxDxxxDDxxxDDxxxDxDDDx',
             'xUUUxUxxxUUxxxUUxxxUxDDDxxDDDxUxxxDUxxxDUxxxDxDDDx',
             'xUUUxUxxxUUxxxUUxxxUxDDDxxDDDxDxxxUDxxxUDxxxUxDDDx',
             'xUUUxUxxxUUxxxUUxxxUxDDDxxDDDxDxxxDDxxxDDxxxDxUUUx',
             'xUUUxUxxxDUxxxDUxxxDxUUUxxUUUxDxxxDDxxxDDxxxDxDDDx',
             'xUUUxUxxxDUxxxDUxxxDxUUUxxDDDxUxxxDUxxxDUxxxDxDDDx',
             'xUUUxUxxxDUxxxDUxxxDxUUUxxDDDxDxxxUDxxxUDxxxUxDDDx',
             'xUUUxUxxxDUxxxDUxxxDxUUUxxDDDxDxxxDDxxxDDxxxDxUUUx',
             'xUUUxUxxxDUxxxDUxxxDxDDDxxUUUxUxxxDUxxxDUxxxDxDDDx',
             'xUUUxUxxxDUxxxDUxxxDxDDDxxUUUxDxxxUDxxxUDxxxUxDDDx',
             'xUUUxUxxxDUxxxDUxxxDxDDDxxUUUxDxxxDDxxxDDxxxDxUUUx',
             'xUUUxUxxxDUxxxDUxxxDxDDDxxDDDxUxxxUUxxxUUxxxUxDDDx',
             'xUUUxUxxxDUxxxDUxxxDxDDDxxDDDxUxxxDUxxxDUxxxDxUUUx',
             'xUUUxUxxxDUxxxDUxxxDxDDDxxDDDxDxxxUDxxxUDxxxUxUUUx',
             'xUUUxDxxxUDxxxUDxxxUxUUUxxUUUxDxxxDDxxxDDxxxDxDDDx',
             'xUUUxDxxxUDxxxUDxxxUxUUUxxDDDxUxxxDUxxxDUxxxDxDDDx',
             'xUUUxDxxxUDxxxUDxxxUxUUUxxDDDxDxxxUDxxxUDxxxUxDDDx',
             'xUUUxDxxxUDxxxUDxxxUxUUUxxDDDxDxxxDDxxxDDxxxDxUUUx',
             'xUUUxDxxxUDxxxUDxxxUxDDDxxUUUxUxxxDUxxxDUxxxDxDDDx',
             'xUUUxDxxxUDxxxUDxxxUxDDDxxUUUxDxxxUDxxxUDxxxUxDDDx',
             'xUUUxDxxxUDxxxUDxxxUxDDDxxUUUxDxxxDDxxxDDxxxDxUUUx',
             'xUUUxDxxxUDxxxUDxxxUxDDDxxDDDxUxxxUUxxxUUxxxUxDDDx',
             'xUUUxDxxxUDxxxUDxxxUxDDDxxDDDxUxxxDUxxxDUxxxDxUUUx',
             'xUUUxDxxxUDxxxUDxxxUxDDDxxDDDxDxxxUDxxxUDxxxUxUUUx',
             'xUUUxDxxxDDxxxDDxxxDxUUUxxUUUxUxxxDUxxxDUxxxDxDDDx',
             'xUUUxDxxxDDxxxDDxxxDxUUUxxUUUxDxxxUDxxxUDxxxUxDDDx',
             'xUUUxDxxxDDxxxDDxxxDxUUUxxUUUxDxxxDDxxxDDxxxDxUUUx',
             'xUUUxDxxxDDxxxDDxxxDxUUUxxDDDxUxxxUUxxxUUxxxUxDDDx',
             'xUUUxDxxxDDxxxDDxxxDxUUUxxDDDxUxxxDUxxxDUxxxDxUUUx',
             'xUUUxDxxxDDxxxDDxxxDxUUUxxDDDxDxxxUDxxxUDxxxUxUUUx',
             'xUUUxDxxxDDxxxDDxxxDxDDDxxUUUxUxxxUUxxxUUxxxUxDDDx',
             'xUUUxDxxxDDxxxDDxxxDxDDDxxUUUxUxxxDUxxxDUxxxDxUUUx',
             'xUUUxDxxxDDxxxDDxxxDxDDDxxUUUxDxxxUDxxxUDxxxUxUUUx',
             'xUUUxDxxxDDxxxDDxxxDxDDDxxDDDxUxxxUUxxxUUxxxUxUUUx',
             'xDDDxUxxxUUxxxUUxxxUxUUUxxUUUxDxxxDDxxxDDxxxDxDDDx',
             'xDDDxUxxxUUxxxUUxxxUxUUUxxDDDxUxxxDUxxxDUxxxDxDDDx',
             'xDDDxUxxxUUxxxUUxxxUxUUUxxDDDxDxxxUDxxxUDxxxUxDDDx',
             'xDDDxUxxxUUxxxUUxxxUxUUUxxDDDxDxxxDDxxxDDxxxDxUUUx',
             'xDDDxUxxxUUxxxUUxxxUxDDDxxUUUxUxxxDUxxxDUxxxDxDDDx',
             'xDDDxUxxxUUxxxUUxxxUxDDDxxUUUxDxxxUDxxxUDxxxUxDDDx',
             'xDDDxUxxxUUxxxUUxxxUxDDDxxUUUxDxxxDDxxxDDxxxDxUUUx',
             'xDDDxUxxxUUxxxUUxxxUxDDDxxDDDxUxxxUUxxxUUxxxUxDDDx',
             'xDDDxUxxxUUxxxUUxxxUxDDDxxDDDxUxxxDUxxxDUxxxDxUUUx',
             'xDDDxUxxxUUxxxUUxxxUxDDDxxDDDxDxxxUDxxxUDxxxUxUUUx',
             'xDDDxUxxxDUxxxDUxxxDxUUUxxUUUxUxxxDUxxxDUxxxDxDDDx',
             'xDDDxUxxxDUxxxDUxxxDxUUUxxUUUxDxxxUDxxxUDxxxUxDDDx',
             'xDDDxUxxxDUxxxDUxxxDxUUUxxUUUxDxxxDDxxxDDxxxDxUUUx',
             'xDDDxUxxxDUxxxDUxxxDxUUUxxDDDxUxxxUUxxxUUxxxUxDDDx',
             'xDDDxUxxxDUxxxDUxxxDxUUUxxDDDxUxxxDUxxxDUxxxDxUUUx',
             'xDDDxUxxxDUxxxDUxxxDxUUUxxDDDxDxxxUDxxxUDxxxUxUUUx',
             'xDDDxUxxxDUxxxDUxxxDxDDDxxUUUxUxxxUUxxxUUxxxUxDDDx',
             'xDDDxUxxxDUxxxDUxxxDxDDDxxUUUxUxxxDUxxxDUxxxDxUUUx',
             'xDDDxUxxxDUxxxDUxxxDxDDDxxUUUxDxxxUDxxxUDxxxUxUUUx',
             'xDDDxUxxxDUxxxDUxxxDxDDDxxDDDxUxxxUUxxxUUxxxUxUUUx',
             'xDDDxDxxxUDxxxUDxxxUxUUUxxUUUxUxxxDUxxxDUxxxDxDDDx',
             'xDDDxDxxxUDxxxUDxxxUxUUUxxUUUxDxxxUDxxxUDxxxUxDDDx',
             'xDDDxDxxxUDxxxUDxxxUxUUUxxUUUxDxxxDDxxxDDxxxDxUUUx',
             'xDDDxDxxxUDxxxUDxxxUxUUUxxDDDxUxxxUUxxxUUxxxUxDDDx',
             'xDDDxDxxxUDxxxUDxxxUxUUUxxDDDxUxxxDUxxxDUxxxDxUUUx',
             'xDDDxDxxxUDxxxUDxxxUxUUUxxDDDxDxxxUDxxxUDxxxUxUUUx',
             'xDDDxDxxxUDxxxUDxxxUxDDDxxUUUxUxxxUUxxxUUxxxUxDDDx',
             'xDDDxDxxxUDxxxUDxxxUxDDDxxUUUxUxxxDUxxxDUxxxDxUUUx',
             'xDDDxDxxxUDxxxUDxxxUxDDDxxUUUxDxxxUDxxxUDxxxUxUUUx',
             'xDDDxDxxxUDxxxUDxxxUxDDDxxDDDxUxxxUUxxxUUxxxUxUUUx',
             'xDDDxDxxxDDxxxDDxxxDxUUUxxUUUxUxxxUUxxxUUxxxUxDDDx',
             'xDDDxDxxxDDxxxDDxxxDxUUUxxUUUxUxxxDUxxxDUxxxDxUUUx',
             'xDDDxDxxxDDxxxDDxxxDxUUUxxUUUxDxxxUDxxxUDxxxUxUUUx',
             'xDDDxDxxxDDxxxDDxxxDxUUUxxDDDxUxxxUUxxxUUxxxUxUUUx',
             'xDDDxDxxxDDxxxDDxxxDxDDDxxUUUxUxxxUUxxxUUxxxUxUUUx'),
            linecount=343000)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Upper
            'x', parent_state[10], parent_state[11], parent_state[12], 'x',
            parent_state[16], 'xxx', parent_state[20],
            parent_state[23], 'xxx', parent_state[27],
            parent_state[30], 'xxx', parent_state[34],
            'x', parent_state[38], parent_state[39], parent_state[40], 'x',

            # Down
            'x', parent_state[255], parent_state[256], parent_state[257], 'x',
            parent_state[261], 'xxx', parent_state[265],
            parent_state[268], 'xxx', parent_state[272],
            parent_state[275], 'xxx', parent_state[279],
            'x', parent_state[283], parent_state[284], parent_state[285], 'x'
        ]

        result = ''.join(result)
        return result


class LookupTableIDA777UDSolveInnerCentersAndObliqueEdges(LookupTableIDA):
    """
    I built this 6-deep for all 70 tables and merged them together into the following

    lookup-table-7x7x7-step50-UD-solve-inner-center-and-oblique-edges.txt
    =====================================================================
    1 steps has 350 entries (0 percent, 0.00x previous step)
    2 steps has 2036 entries (0 percent, 5.82x previous step)
    3 steps has 13108 entries (0 percent, 6.44x previous step)
    4 steps has 86624 entries (2 percent, 6.61x previous step)
    5 steps has 560132 entries (13 percent, 6.47x previous step)
    6 steps has 3456952 entries (83 percent, 6.17x previous step)

    Total: 4119202 entries
    """

    def __init__(self, parent):
        LookupTableIDA.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step50-UD-solve-inner-center-and-oblique-edges.txt',
            ('xUUUxUUUUUUUUUUUUUUUxUUUxxDDDxDDDDDDDDDDDDDDDxDDDx',
             'xUUUxUUUUUUUUUUUUUUUxDDDxxUUUxDDDDDDDDDDDDDDDxDDDx',
             'xUUUxUUUUUUUUUUUUUUUxDDDxxDDDxUDDDDUDDDDUDDDDxDDDx',
             'xUUUxUUUUUUUUUUUUUUUxDDDxxDDDxDDDDUDDDDUDDDDUxDDDx',
             'xUUUxUUUUUUUUUUUUUUUxDDDxxDDDxDDDDDDDDDDDDDDDxUUUx',
             'xUUUxUUUUDUUUUDUUUUDxUUUxxUUUxDDDDDDDDDDDDDDDxDDDx',
             'xUUUxUUUUDUUUUDUUUUDxUUUxxDDDxUDDDDUDDDDUDDDDxDDDx',
             'xUUUxUUUUDUUUUDUUUUDxUUUxxDDDxDDDDUDDDDUDDDDUxDDDx',
             'xUUUxUUUUDUUUUDUUUUDxUUUxxDDDxDDDDDDDDDDDDDDDxUUUx',
             'xUUUxUUUUDUUUUDUUUUDxDDDxxUUUxUDDDDUDDDDUDDDDxDDDx',
             'xUUUxUUUUDUUUUDUUUUDxDDDxxUUUxDDDDUDDDDUDDDDUxDDDx',
             'xUUUxUUUUDUUUUDUUUUDxDDDxxUUUxDDDDDDDDDDDDDDDxUUUx',
             'xUUUxUUUUDUUUUDUUUUDxDDDxxDDDxUDDDUUDDDUUDDDUxDDDx',
             'xUUUxUUUUDUUUUDUUUUDxDDDxxDDDxUDDDDUDDDDUDDDDxUUUx',
             'xUUUxUUUUDUUUUDUUUUDxDDDxxDDDxDDDDUDDDDUDDDDUxUUUx',
             'xUUUxDUUUUDUUUUDUUUUxUUUxxUUUxDDDDDDDDDDDDDDDxDDDx',
             'xUUUxDUUUUDUUUUDUUUUxUUUxxDDDxUDDDDUDDDDUDDDDxDDDx',
             'xUUUxDUUUUDUUUUDUUUUxUUUxxDDDxDDDDUDDDDUDDDDUxDDDx',
             'xUUUxDUUUUDUUUUDUUUUxUUUxxDDDxDDDDDDDDDDDDDDDxUUUx',
             'xUUUxDUUUUDUUUUDUUUUxDDDxxUUUxUDDDDUDDDDUDDDDxDDDx',
             'xUUUxDUUUUDUUUUDUUUUxDDDxxUUUxDDDDUDDDDUDDDDUxDDDx',
             'xUUUxDUUUUDUUUUDUUUUxDDDxxUUUxDDDDDDDDDDDDDDDxUUUx',
             'xUUUxDUUUUDUUUUDUUUUxDDDxxDDDxUDDDUUDDDUUDDDUxDDDx',
             'xUUUxDUUUUDUUUUDUUUUxDDDxxDDDxUDDDDUDDDDUDDDDxUUUx',
             'xUUUxDUUUUDUUUUDUUUUxDDDxxDDDxDDDDUDDDDUDDDDUxUUUx',
             'xUUUxDUUUDDUUUDDUUUDxUUUxxUUUxUDDDDUDDDDUDDDDxDDDx',
             'xUUUxDUUUDDUUUDDUUUDxUUUxxUUUxDDDDUDDDDUDDDDUxDDDx',
             'xUUUxDUUUDDUUUDDUUUDxUUUxxUUUxDDDDDDDDDDDDDDDxUUUx',
             'xUUUxDUUUDDUUUDDUUUDxUUUxxDDDxUDDDUUDDDUUDDDUxDDDx',
             'xUUUxDUUUDDUUUDDUUUDxUUUxxDDDxUDDDDUDDDDUDDDDxUUUx',
             'xUUUxDUUUDDUUUDDUUUDxUUUxxDDDxDDDDUDDDDUDDDDUxUUUx',
             'xUUUxDUUUDDUUUDDUUUDxDDDxxUUUxUDDDUUDDDUUDDDUxDDDx',
             'xUUUxDUUUDDUUUDDUUUDxDDDxxUUUxUDDDDUDDDDUDDDDxUUUx',
             'xUUUxDUUUDDUUUDDUUUDxDDDxxUUUxDDDDUDDDDUDDDDUxUUUx',
             'xUUUxDUUUDDUUUDDUUUDxDDDxxDDDxUDDDUUDDDUUDDDUxUUUx',
             'xDDDxUUUUUUUUUUUUUUUxUUUxxUUUxDDDDDDDDDDDDDDDxDDDx',
             'xDDDxUUUUUUUUUUUUUUUxUUUxxDDDxUDDDDUDDDDUDDDDxDDDx',
             'xDDDxUUUUUUUUUUUUUUUxUUUxxDDDxDDDDUDDDDUDDDDUxDDDx',
             'xDDDxUUUUUUUUUUUUUUUxUUUxxDDDxDDDDDDDDDDDDDDDxUUUx',
             'xDDDxUUUUUUUUUUUUUUUxDDDxxUUUxUDDDDUDDDDUDDDDxDDDx',
             'xDDDxUUUUUUUUUUUUUUUxDDDxxUUUxDDDDUDDDDUDDDDUxDDDx',
             'xDDDxUUUUUUUUUUUUUUUxDDDxxUUUxDDDDDDDDDDDDDDDxUUUx',
             'xDDDxUUUUUUUUUUUUUUUxDDDxxDDDxUDDDUUDDDUUDDDUxDDDx',
             'xDDDxUUUUUUUUUUUUUUUxDDDxxDDDxUDDDDUDDDDUDDDDxUUUx',
             'xDDDxUUUUUUUUUUUUUUUxDDDxxDDDxDDDDUDDDDUDDDDUxUUUx',
             'xDDDxUUUUDUUUUDUUUUDxUUUxxUUUxUDDDDUDDDDUDDDDxDDDx',
             'xDDDxUUUUDUUUUDUUUUDxUUUxxUUUxDDDDUDDDDUDDDDUxDDDx',
             'xDDDxUUUUDUUUUDUUUUDxUUUxxUUUxDDDDDDDDDDDDDDDxUUUx',
             'xDDDxUUUUDUUUUDUUUUDxUUUxxDDDxUDDDUUDDDUUDDDUxDDDx',
             'xDDDxUUUUDUUUUDUUUUDxUUUxxDDDxUDDDDUDDDDUDDDDxUUUx',
             'xDDDxUUUUDUUUUDUUUUDxUUUxxDDDxDDDDUDDDDUDDDDUxUUUx',
             'xDDDxUUUUDUUUUDUUUUDxDDDxxUUUxUDDDUUDDDUUDDDUxDDDx',
             'xDDDxUUUUDUUUUDUUUUDxDDDxxUUUxUDDDDUDDDDUDDDDxUUUx',
             'xDDDxUUUUDUUUUDUUUUDxDDDxxUUUxDDDDUDDDDUDDDDUxUUUx',
             'xDDDxUUUUDUUUUDUUUUDxDDDxxDDDxUDDDUUDDDUUDDDUxUUUx',
             'xDDDxDUUUUDUUUUDUUUUxUUUxxUUUxUDDDDUDDDDUDDDDxDDDx',
             'xDDDxDUUUUDUUUUDUUUUxUUUxxUUUxDDDDUDDDDUDDDDUxDDDx',
             'xDDDxDUUUUDUUUUDUUUUxUUUxxUUUxDDDDDDDDDDDDDDDxUUUx',
             'xDDDxDUUUUDUUUUDUUUUxUUUxxDDDxUDDDUUDDDUUDDDUxDDDx',
             'xDDDxDUUUUDUUUUDUUUUxUUUxxDDDxUDDDDUDDDDUDDDDxUUUx',
             'xDDDxDUUUUDUUUUDUUUUxUUUxxDDDxDDDDUDDDDUDDDDUxUUUx',
             'xDDDxDUUUUDUUUUDUUUUxDDDxxUUUxUDDDUUDDDUUDDDUxDDDx',
             'xDDDxDUUUUDUUUUDUUUUxDDDxxUUUxUDDDDUDDDDUDDDDxUUUx',
             'xDDDxDUUUUDUUUUDUUUUxDDDxxUUUxDDDDUDDDDUDDDDUxUUUx',
             'xDDDxDUUUUDUUUUDUUUUxDDDxxDDDxUDDDUUDDDUUDDDUxUUUx',
             'xDDDxDUUUDDUUUDDUUUDxUUUxxUUUxUDDDUUDDDUUDDDUxDDDx',
             'xDDDxDUUUDDUUUDDUUUDxUUUxxUUUxUDDDDUDDDDUDDDDxUUUx',
             'xDDDxDUUUDDUUUDDUUUDxUUUxxUUUxDDDDUDDDDUDDDDUxUUUx',
             'xDDDxDUUUDDUUUDDUUUDxUUUxxDDDxUDDDUUDDDUUDDDUxUUUx',
             'xDDDxDUUUDDUUUDDUUUDxDDDxxUUUxUDDDUUDDDUUDDDUxUUUx'),
            moves_7x7x7,

            ("3Rw", "3Rw'", "3Lw", "3Lw'", "3Fw", "3Fw'", "3Bw", "3Bw'", "3Uw", "3Uw'", "3Dw", "3Dw'", # do not mess up staged centers
             "Rw", "Rw'", "Lw", "Lw'", "Fw", "Fw'", "Bw", "Bw'", "Uw", "Uw'", "Dw", "Dw'"),            # do not mess up staged centers

            # prune tables
            (parent.lt_UD_solve_inner_centers_and_oblique_edges_center_only,
             parent.lt_UD_solve_inner_centers_and_oblique_edges_edges_only),
            linecount=4119202)

    def state(self):
        parent_state = self.parent.state
        result = [
            # Upper
            'x', parent_state[10], parent_state[11], parent_state[12], 'x',
            parent_state[16], parent_state[17], parent_state[18], parent_state[19], parent_state[20],
            parent_state[23], parent_state[24], parent_state[25], parent_state[26], parent_state[27],
            parent_state[30], parent_state[31], parent_state[32], parent_state[33], parent_state[34],
            'x', parent_state[38], parent_state[39], parent_state[40], 'x',

            # Down
            'x', parent_state[255], parent_state[256], parent_state[257], 'x',
            parent_state[261], parent_state[262], parent_state[263], parent_state[264], parent_state[265],
            parent_state[268], parent_state[269], parent_state[270], parent_state[271], parent_state[272],
            parent_state[275], parent_state[276], parent_state[277], parent_state[278], parent_state[279],
            'x', parent_state[283], parent_state[284], parent_state[285], 'x'
        ]

        result = ''.join(result)
        return result


class LookupTable777LRSolveInnerXCenterTCenterMiddleObliqueEdge(LookupTable):
    """
    lookup-table-7x7x7-step61-LR-inner-x-center-t-center-and-middle-oblique-edges.txt
    ==================================================================================
    1 steps has 210 entries (0 percent, 0.00x previous step)
    2 steps has 770 entries (0 percent, 3.67x previous step)
    3 steps has 1,540 entries (0 percent, 2.00x previous step)
    4 steps has 5,950 entries (1 percent, 3.86x previous step)
    5 steps has 15,400 entries (4 percent, 2.59x previous step)
    6 steps has 44,310 entries (12 percent, 2.88x previous step)
    7 steps has 82,600 entries (24 percent, 1.86x previous step)
    8 steps has 120,960 entries (35 percent, 1.46x previous step)
    9 steps has 62,160 entries (18 percent, 0.51x previous step)
    10 steps has 8,820 entries (2 percent, 0.14x previous step)
    11 steps has 280 entries (0 percent, 0.03x previous step)

    Total: 343,000 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step61-LR-inner-x-center-t-center-and-middle-oblique-edges.txt',
            ('xxLxxxLLLxLLLLLxLLLxxxLxxxxRxxxRRRxRRRRRxRRRxxxRxx',
             'xxLxxxLLLxLLLLLxLLLxxxRxxxxLxxxRRRxRRRRRxRRRxxxRxx',
             'xxLxxxLLLxLLLLLxLLLxxxRxxxxRxxxRRRxLRRRRxRRRxxxRxx',
             'xxLxxxLLLxLLLLLxLLLxxxRxxxxRxxxRRRxRRRRLxRRRxxxRxx',
             'xxLxxxLLLxLLLLLxLLLxxxRxxxxRxxxRRRxRRRRRxRRRxxxLxx',
             'xxLxxxLLLxLLLLRxLLLxxxLxxxxLxxxRRRxRRRRRxRRRxxxRxx',
             'xxLxxxLLLxLLLLRxLLLxxxLxxxxRxxxRRRxLRRRRxRRRxxxRxx',
             'xxLxxxLLLxLLLLRxLLLxxxLxxxxRxxxRRRxRRRRLxRRRxxxRxx',
             'xxLxxxLLLxLLLLRxLLLxxxLxxxxRxxxRRRxRRRRRxRRRxxxLxx',
             'xxLxxxLLLxLLLLRxLLLxxxRxxxxLxxxRRRxLRRRRxRRRxxxRxx',
             'xxLxxxLLLxLLLLRxLLLxxxRxxxxLxxxRRRxRRRRLxRRRxxxRxx',
             'xxLxxxLLLxLLLLRxLLLxxxRxxxxLxxxRRRxRRRRRxRRRxxxLxx',
             'xxLxxxLLLxLLLLRxLLLxxxRxxxxRxxxRRRxLRRRLxRRRxxxRxx',
             'xxLxxxLLLxLLLLRxLLLxxxRxxxxRxxxRRRxLRRRRxRRRxxxLxx',
             'xxLxxxLLLxLLLLRxLLLxxxRxxxxRxxxRRRxRRRRLxRRRxxxLxx',
             'xxLxxxLLLxRLLLLxLLLxxxLxxxxLxxxRRRxRRRRRxRRRxxxRxx',
             'xxLxxxLLLxRLLLLxLLLxxxLxxxxRxxxRRRxLRRRRxRRRxxxRxx',
             'xxLxxxLLLxRLLLLxLLLxxxLxxxxRxxxRRRxRRRRLxRRRxxxRxx',
             'xxLxxxLLLxRLLLLxLLLxxxLxxxxRxxxRRRxRRRRRxRRRxxxLxx',
             'xxLxxxLLLxRLLLLxLLLxxxRxxxxLxxxRRRxLRRRRxRRRxxxRxx',
             'xxLxxxLLLxRLLLLxLLLxxxRxxxxLxxxRRRxRRRRLxRRRxxxRxx',
             'xxLxxxLLLxRLLLLxLLLxxxRxxxxLxxxRRRxRRRRRxRRRxxxLxx',
             'xxLxxxLLLxRLLLLxLLLxxxRxxxxRxxxRRRxLRRRLxRRRxxxRxx',
             'xxLxxxLLLxRLLLLxLLLxxxRxxxxRxxxRRRxLRRRRxRRRxxxLxx',
             'xxLxxxLLLxRLLLLxLLLxxxRxxxxRxxxRRRxRRRRLxRRRxxxLxx',
             'xxLxxxLLLxRLLLRxLLLxxxLxxxxLxxxRRRxLRRRRxRRRxxxRxx',
             'xxLxxxLLLxRLLLRxLLLxxxLxxxxLxxxRRRxRRRRLxRRRxxxRxx',
             'xxLxxxLLLxRLLLRxLLLxxxLxxxxLxxxRRRxRRRRRxRRRxxxLxx',
             'xxLxxxLLLxRLLLRxLLLxxxLxxxxRxxxRRRxLRRRLxRRRxxxRxx',
             'xxLxxxLLLxRLLLRxLLLxxxLxxxxRxxxRRRxLRRRRxRRRxxxLxx',
             'xxLxxxLLLxRLLLRxLLLxxxLxxxxRxxxRRRxRRRRLxRRRxxxLxx',
             'xxLxxxLLLxRLLLRxLLLxxxRxxxxLxxxRRRxLRRRLxRRRxxxRxx',
             'xxLxxxLLLxRLLLRxLLLxxxRxxxxLxxxRRRxLRRRRxRRRxxxLxx',
             'xxLxxxLLLxRLLLRxLLLxxxRxxxxLxxxRRRxRRRRLxRRRxxxLxx',
             'xxLxxxLLLxRLLLRxLLLxxxRxxxxRxxxRRRxLRRRLxRRRxxxLxx',
             'xxRxxxLLLxLLLLLxLLLxxxLxxxxLxxxRRRxRRRRRxRRRxxxRxx',
             'xxRxxxLLLxLLLLLxLLLxxxLxxxxRxxxRRRxLRRRRxRRRxxxRxx',
             'xxRxxxLLLxLLLLLxLLLxxxLxxxxRxxxRRRxRRRRLxRRRxxxRxx',
             'xxRxxxLLLxLLLLLxLLLxxxLxxxxRxxxRRRxRRRRRxRRRxxxLxx',
             'xxRxxxLLLxLLLLLxLLLxxxRxxxxLxxxRRRxLRRRRxRRRxxxRxx',
             'xxRxxxLLLxLLLLLxLLLxxxRxxxxLxxxRRRxRRRRLxRRRxxxRxx',
             'xxRxxxLLLxLLLLLxLLLxxxRxxxxLxxxRRRxRRRRRxRRRxxxLxx',
             'xxRxxxLLLxLLLLLxLLLxxxRxxxxRxxxRRRxLRRRLxRRRxxxRxx',
             'xxRxxxLLLxLLLLLxLLLxxxRxxxxRxxxRRRxLRRRRxRRRxxxLxx',
             'xxRxxxLLLxLLLLLxLLLxxxRxxxxRxxxRRRxRRRRLxRRRxxxLxx',
             'xxRxxxLLLxLLLLRxLLLxxxLxxxxLxxxRRRxLRRRRxRRRxxxRxx',
             'xxRxxxLLLxLLLLRxLLLxxxLxxxxLxxxRRRxRRRRLxRRRxxxRxx',
             'xxRxxxLLLxLLLLRxLLLxxxLxxxxLxxxRRRxRRRRRxRRRxxxLxx',
             'xxRxxxLLLxLLLLRxLLLxxxLxxxxRxxxRRRxLRRRLxRRRxxxRxx',
             'xxRxxxLLLxLLLLRxLLLxxxLxxxxRxxxRRRxLRRRRxRRRxxxLxx',
             'xxRxxxLLLxLLLLRxLLLxxxLxxxxRxxxRRRxRRRRLxRRRxxxLxx',
             'xxRxxxLLLxLLLLRxLLLxxxRxxxxLxxxRRRxLRRRLxRRRxxxRxx',
             'xxRxxxLLLxLLLLRxLLLxxxRxxxxLxxxRRRxLRRRRxRRRxxxLxx',
             'xxRxxxLLLxLLLLRxLLLxxxRxxxxLxxxRRRxRRRRLxRRRxxxLxx',
             'xxRxxxLLLxLLLLRxLLLxxxRxxxxRxxxRRRxLRRRLxRRRxxxLxx',
             'xxRxxxLLLxRLLLLxLLLxxxLxxxxLxxxRRRxLRRRRxRRRxxxRxx',
             'xxRxxxLLLxRLLLLxLLLxxxLxxxxLxxxRRRxRRRRLxRRRxxxRxx',
             'xxRxxxLLLxRLLLLxLLLxxxLxxxxLxxxRRRxRRRRRxRRRxxxLxx',
             'xxRxxxLLLxRLLLLxLLLxxxLxxxxRxxxRRRxLRRRLxRRRxxxRxx',
             'xxRxxxLLLxRLLLLxLLLxxxLxxxxRxxxRRRxLRRRRxRRRxxxLxx',
             'xxRxxxLLLxRLLLLxLLLxxxLxxxxRxxxRRRxRRRRLxRRRxxxLxx',
             'xxRxxxLLLxRLLLLxLLLxxxRxxxxLxxxRRRxLRRRLxRRRxxxRxx',
             'xxRxxxLLLxRLLLLxLLLxxxRxxxxLxxxRRRxLRRRRxRRRxxxLxx',
             'xxRxxxLLLxRLLLLxLLLxxxRxxxxLxxxRRRxRRRRLxRRRxxxLxx',
             'xxRxxxLLLxRLLLLxLLLxxxRxxxxRxxxRRRxLRRRLxRRRxxxLxx',
             'xxRxxxLLLxRLLLRxLLLxxxLxxxxLxxxRRRxLRRRLxRRRxxxRxx',
             'xxRxxxLLLxRLLLRxLLLxxxLxxxxLxxxRRRxLRRRRxRRRxxxLxx',
             'xxRxxxLLLxRLLLRxLLLxxxLxxxxLxxxRRRxRRRRLxRRRxxxLxx',
             'xxRxxxLLLxRLLLRxLLLxxxLxxxxRxxxRRRxLRRRLxRRRxxxLxx',
             'xxRxxxLLLxRLLLRxLLLxxxRxxxxLxxxRRRxLRRRLxRRRxxxLxx'),
            linecount=343000)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Left
            'xx', parent_state[60], 'xx',
            'x', parent_state[66], parent_state[67], parent_state[68], 'x',
            parent_state[72], parent_state[73], parent_state[74], parent_state[75], parent_state[76],
            'x', parent_state[80], parent_state[81], parent_state[82], 'x',
            'xx', parent_state[88], 'xx',

            # Right
            'xx', parent_state[158], 'xx',
            'x', parent_state[164], parent_state[165], parent_state[166], 'x',
            parent_state[170], parent_state[171], parent_state[172], parent_state[173], parent_state[174],
            'x', parent_state[178], parent_state[179], parent_state[180], 'x',
            'xx', parent_state[186], 'xx'
        ]

        result = ['x' if x in ('U', 'D', 'F', 'B') else x for x in result]
        result = ''.join(result)

        return result


class LookupTable777LRSolveObliqueEdge(LookupTable):
    """
    lookup-table-7x7x7-step62-LR-oblique-edges.txt
    ==============================================
    1 steps has 182 entries (0 percent, 0.00x previous step)
    2 steps has 616 entries (0 percent, 3.38x previous step)
    3 steps has 2,044 entries (0 percent, 3.32x previous step)
    4 steps has 8,576 entries (2 percent, 4.20x previous step)
    5 steps has 21,516 entries (6 percent, 2.51x previous step)
    6 steps has 60,392 entries (17 percent, 2.81x previous step)
    7 steps has 105,050 entries (30 percent, 1.74x previous step)
    8 steps has 106,464 entries (31 percent, 1.01x previous step)
    9 steps has 35,772 entries (10 percent, 0.34x previous step)
    10 steps has 2,368 entries (0 percent, 0.07x previous step)
    11 steps has 20 entries (0 percent, 0.01x previous step)

    Total: 343,000 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step62-LR-oblique-edges.txt',
            ('xLLLxLxxxLLxxxLLxxxLxLLLxxRRRxRxxxRRxxxRRxxxRxRRRx',
             'xLLLxLxxxLLxxxLLxxxLxRRRxxLLLxRxxxRRxxxRRxxxRxRRRx',
             'xLLLxLxxxLLxxxLLxxxLxRRRxxRRRxLxxxRLxxxRLxxxRxRRRx',
             'xLLLxLxxxLLxxxLLxxxLxRRRxxRRRxRxxxLRxxxLRxxxLxRRRx',
             'xLLLxLxxxLLxxxLLxxxLxRRRxxRRRxRxxxRRxxxRRxxxRxLLLx',
             'xLLLxLxxxRLxxxRLxxxRxLLLxxLLLxRxxxRRxxxRRxxxRxRRRx',
             'xLLLxLxxxRLxxxRLxxxRxLLLxxRRRxLxxxRLxxxRLxxxRxRRRx',
             'xLLLxLxxxRLxxxRLxxxRxLLLxxRRRxRxxxLRxxxLRxxxLxRRRx',
             'xLLLxLxxxRLxxxRLxxxRxLLLxxRRRxRxxxRRxxxRRxxxRxLLLx',
             'xLLLxLxxxRLxxxRLxxxRxRRRxxLLLxLxxxRLxxxRLxxxRxRRRx',
             'xLLLxLxxxRLxxxRLxxxRxRRRxxLLLxRxxxLRxxxLRxxxLxRRRx',
             'xLLLxLxxxRLxxxRLxxxRxRRRxxLLLxRxxxRRxxxRRxxxRxLLLx',
             'xLLLxLxxxRLxxxRLxxxRxRRRxxRRRxLxxxLLxxxLLxxxLxRRRx',
             'xLLLxLxxxRLxxxRLxxxRxRRRxxRRRxLxxxRLxxxRLxxxRxLLLx',
             'xLLLxLxxxRLxxxRLxxxRxRRRxxRRRxRxxxLRxxxLRxxxLxLLLx',
             'xLLLxRxxxLRxxxLRxxxLxLLLxxLLLxRxxxRRxxxRRxxxRxRRRx',
             'xLLLxRxxxLRxxxLRxxxLxLLLxxRRRxLxxxRLxxxRLxxxRxRRRx',
             'xLLLxRxxxLRxxxLRxxxLxLLLxxRRRxRxxxLRxxxLRxxxLxRRRx',
             'xLLLxRxxxLRxxxLRxxxLxLLLxxRRRxRxxxRRxxxRRxxxRxLLLx',
             'xLLLxRxxxLRxxxLRxxxLxRRRxxLLLxLxxxRLxxxRLxxxRxRRRx',
             'xLLLxRxxxLRxxxLRxxxLxRRRxxLLLxRxxxLRxxxLRxxxLxRRRx',
             'xLLLxRxxxLRxxxLRxxxLxRRRxxLLLxRxxxRRxxxRRxxxRxLLLx',
             'xLLLxRxxxLRxxxLRxxxLxRRRxxRRRxLxxxLLxxxLLxxxLxRRRx',
             'xLLLxRxxxLRxxxLRxxxLxRRRxxRRRxLxxxRLxxxRLxxxRxLLLx',
             'xLLLxRxxxLRxxxLRxxxLxRRRxxRRRxRxxxLRxxxLRxxxLxLLLx',
             'xLLLxRxxxRRxxxRRxxxRxLLLxxLLLxLxxxRLxxxRLxxxRxRRRx',
             'xLLLxRxxxRRxxxRRxxxRxLLLxxLLLxRxxxLRxxxLRxxxLxRRRx',
             'xLLLxRxxxRRxxxRRxxxRxLLLxxLLLxRxxxRRxxxRRxxxRxLLLx',
             'xLLLxRxxxRRxxxRRxxxRxLLLxxRRRxLxxxLLxxxLLxxxLxRRRx',
             'xLLLxRxxxRRxxxRRxxxRxLLLxxRRRxLxxxRLxxxRLxxxRxLLLx',
             'xLLLxRxxxRRxxxRRxxxRxLLLxxRRRxRxxxLRxxxLRxxxLxLLLx',
             'xLLLxRxxxRRxxxRRxxxRxRRRxxLLLxLxxxLLxxxLLxxxLxRRRx',
             'xLLLxRxxxRRxxxRRxxxRxRRRxxLLLxLxxxRLxxxRLxxxRxLLLx',
             'xLLLxRxxxRRxxxRRxxxRxRRRxxLLLxRxxxLRxxxLRxxxLxLLLx',
             'xLLLxRxxxRRxxxRRxxxRxRRRxxRRRxLxxxLLxxxLLxxxLxLLLx',
             'xRRRxLxxxLLxxxLLxxxLxLLLxxLLLxRxxxRRxxxRRxxxRxRRRx',
             'xRRRxLxxxLLxxxLLxxxLxLLLxxRRRxLxxxRLxxxRLxxxRxRRRx',
             'xRRRxLxxxLLxxxLLxxxLxLLLxxRRRxRxxxLRxxxLRxxxLxRRRx',
             'xRRRxLxxxLLxxxLLxxxLxLLLxxRRRxRxxxRRxxxRRxxxRxLLLx',
             'xRRRxLxxxLLxxxLLxxxLxRRRxxLLLxLxxxRLxxxRLxxxRxRRRx',
             'xRRRxLxxxLLxxxLLxxxLxRRRxxLLLxRxxxLRxxxLRxxxLxRRRx',
             'xRRRxLxxxLLxxxLLxxxLxRRRxxLLLxRxxxRRxxxRRxxxRxLLLx',
             'xRRRxLxxxLLxxxLLxxxLxRRRxxRRRxLxxxLLxxxLLxxxLxRRRx',
             'xRRRxLxxxLLxxxLLxxxLxRRRxxRRRxLxxxRLxxxRLxxxRxLLLx',
             'xRRRxLxxxLLxxxLLxxxLxRRRxxRRRxRxxxLRxxxLRxxxLxLLLx',
             'xRRRxLxxxRLxxxRLxxxRxLLLxxLLLxLxxxRLxxxRLxxxRxRRRx',
             'xRRRxLxxxRLxxxRLxxxRxLLLxxLLLxRxxxLRxxxLRxxxLxRRRx',
             'xRRRxLxxxRLxxxRLxxxRxLLLxxLLLxRxxxRRxxxRRxxxRxLLLx',
             'xRRRxLxxxRLxxxRLxxxRxLLLxxRRRxLxxxLLxxxLLxxxLxRRRx',
             'xRRRxLxxxRLxxxRLxxxRxLLLxxRRRxLxxxRLxxxRLxxxRxLLLx',
             'xRRRxLxxxRLxxxRLxxxRxLLLxxRRRxRxxxLRxxxLRxxxLxLLLx',
             'xRRRxLxxxRLxxxRLxxxRxRRRxxLLLxLxxxLLxxxLLxxxLxRRRx',
             'xRRRxLxxxRLxxxRLxxxRxRRRxxLLLxLxxxRLxxxRLxxxRxLLLx',
             'xRRRxLxxxRLxxxRLxxxRxRRRxxLLLxRxxxLRxxxLRxxxLxLLLx',
             'xRRRxLxxxRLxxxRLxxxRxRRRxxRRRxLxxxLLxxxLLxxxLxLLLx',
             'xRRRxRxxxLRxxxLRxxxLxLLLxxLLLxLxxxRLxxxRLxxxRxRRRx',
             'xRRRxRxxxLRxxxLRxxxLxLLLxxLLLxRxxxLRxxxLRxxxLxRRRx',
             'xRRRxRxxxLRxxxLRxxxLxLLLxxLLLxRxxxRRxxxRRxxxRxLLLx',
             'xRRRxRxxxLRxxxLRxxxLxLLLxxRRRxLxxxLLxxxLLxxxLxRRRx',
             'xRRRxRxxxLRxxxLRxxxLxLLLxxRRRxLxxxRLxxxRLxxxRxLLLx',
             'xRRRxRxxxLRxxxLRxxxLxLLLxxRRRxRxxxLRxxxLRxxxLxLLLx',
             'xRRRxRxxxLRxxxLRxxxLxRRRxxLLLxLxxxLLxxxLLxxxLxRRRx',
             'xRRRxRxxxLRxxxLRxxxLxRRRxxLLLxLxxxRLxxxRLxxxRxLLLx',
             'xRRRxRxxxLRxxxLRxxxLxRRRxxLLLxRxxxLRxxxLRxxxLxLLLx',
             'xRRRxRxxxLRxxxLRxxxLxRRRxxRRRxLxxxLLxxxLLxxxLxLLLx',
             'xRRRxRxxxRRxxxRRxxxRxLLLxxLLLxLxxxLLxxxLLxxxLxRRRx',
             'xRRRxRxxxRRxxxRRxxxRxLLLxxLLLxLxxxRLxxxRLxxxRxLLLx',
             'xRRRxRxxxRRxxxRRxxxRxLLLxxLLLxRxxxLRxxxLRxxxLxLLLx',
             'xRRRxRxxxRRxxxRRxxxRxLLLxxRRRxLxxxLLxxxLLxxxLxLLLx',
             'xRRRxRxxxRRxxxRRxxxRxRRRxxLLLxLxxxLLxxxLLxxxLxLLLx'),
            linecount=343000)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Left
            'x', parent_state[59], parent_state[60], parent_state[61], 'x',
            parent_state[65], 'xxx', parent_state[69],
            parent_state[72], 'xxx', parent_state[76],
            parent_state[79], 'xxx', parent_state[83],
            'x', parent_state[87], parent_state[88], parent_state[89], 'x',

            # Right
            'x', parent_state[157], parent_state[158], parent_state[159], 'x',
            parent_state[163], 'xxx', parent_state[167],
            parent_state[170], 'xxx', parent_state[174],
            parent_state[177], 'xxx', parent_state[181],
            'x', parent_state[185], parent_state[186], parent_state[187], 'x'
        ]

        result = ['x' if x in ('U', 'D', 'F', 'B') else x for x in result]
        result = ''.join(result)

        return result


class LookupTableIDA777LRSolveInnerCentersAndObliqueEdges(LookupTableIDA):
    """
    lookup-table-7x7x7-step60-LR-solve-inner-center-and-oblique-edges.txt
    =====================================================================
    1 steps has 210 entries (0 percent, 0.00x previous step)
    2 steps has 770 entries (0 percent, 3.67x previous step)
    3 steps has 2,828 entries (0 percent, 3.67x previous step)
    4 steps has 15,158 entries (0 percent, 5.36x previous step)
    5 steps has 63,008 entries (0 percent, 4.16x previous step)
    6 steps has 290,588 entries (4 percent, 4.61x previous step)
    7 steps has 1,232,594 entries (17 percent, 4.24x previous step)
    8 steps has 5,266,642 entries (76 percent, 4.27x previous step)

    Total: 6,871,798 entries
    """

    def __init__(self, parent):
        LookupTableIDA.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step60-LR-solve-inner-center-and-oblique-edges.txt',
            ('xLLLxLLLLLLLLLLLLLLLxLLLxxRRRxRRRRRRRRRRRRRRRxRRRx',
             'xLLLxLLLLLLLLLLLLLLLxRRRxxLLLxRRRRRRRRRRRRRRRxRRRx',
             'xLLLxLLLLLLLLLLLLLLLxRRRxxRRRxLRRRRLRRRRLRRRRxRRRx',
             'xLLLxLLLLLLLLLLLLLLLxRRRxxRRRxRRRRLRRRRLRRRRLxRRRx',
             'xLLLxLLLLLLLLLLLLLLLxRRRxxRRRxRRRRRRRRRRRRRRRxLLLx',
             'xLLLxLLLLRLLLLRLLLLRxLLLxxLLLxRRRRRRRRRRRRRRRxRRRx',
             'xLLLxLLLLRLLLLRLLLLRxLLLxxRRRxLRRRRLRRRRLRRRRxRRRx',
             'xLLLxLLLLRLLLLRLLLLRxLLLxxRRRxRRRRLRRRRLRRRRLxRRRx',
             'xLLLxLLLLRLLLLRLLLLRxLLLxxRRRxRRRRRRRRRRRRRRRxLLLx',
             'xLLLxLLLLRLLLLRLLLLRxRRRxxLLLxLRRRRLRRRRLRRRRxRRRx',
             'xLLLxLLLLRLLLLRLLLLRxRRRxxLLLxRRRRLRRRRLRRRRLxRRRx',
             'xLLLxLLLLRLLLLRLLLLRxRRRxxLLLxRRRRRRRRRRRRRRRxLLLx',
             'xLLLxLLLLRLLLLRLLLLRxRRRxxRRRxLRRRLLRRRLLRRRLxRRRx',
             'xLLLxLLLLRLLLLRLLLLRxRRRxxRRRxLRRRRLRRRRLRRRRxLLLx',
             'xLLLxLLLLRLLLLRLLLLRxRRRxxRRRxRRRRLRRRRLRRRRLxLLLx',
             'xLLLxRLLLLRLLLLRLLLLxLLLxxLLLxRRRRRRRRRRRRRRRxRRRx',
             'xLLLxRLLLLRLLLLRLLLLxLLLxxRRRxLRRRRLRRRRLRRRRxRRRx',
             'xLLLxRLLLLRLLLLRLLLLxLLLxxRRRxRRRRLRRRRLRRRRLxRRRx',
             'xLLLxRLLLLRLLLLRLLLLxLLLxxRRRxRRRRRRRRRRRRRRRxLLLx',
             'xLLLxRLLLLRLLLLRLLLLxRRRxxLLLxLRRRRLRRRRLRRRRxRRRx',
             'xLLLxRLLLLRLLLLRLLLLxRRRxxLLLxRRRRLRRRRLRRRRLxRRRx',
             'xLLLxRLLLLRLLLLRLLLLxRRRxxLLLxRRRRRRRRRRRRRRRxLLLx',
             'xLLLxRLLLLRLLLLRLLLLxRRRxxRRRxLRRRLLRRRLLRRRLxRRRx',
             'xLLLxRLLLLRLLLLRLLLLxRRRxxRRRxLRRRRLRRRRLRRRRxLLLx',
             'xLLLxRLLLLRLLLLRLLLLxRRRxxRRRxRRRRLRRRRLRRRRLxLLLx',
             'xLLLxRLLLRRLLLRRLLLRxLLLxxLLLxLRRRRLRRRRLRRRRxRRRx',
             'xLLLxRLLLRRLLLRRLLLRxLLLxxLLLxRRRRLRRRRLRRRRLxRRRx',
             'xLLLxRLLLRRLLLRRLLLRxLLLxxLLLxRRRRRRRRRRRRRRRxLLLx',
             'xLLLxRLLLRRLLLRRLLLRxLLLxxRRRxLRRRLLRRRLLRRRLxRRRx',
             'xLLLxRLLLRRLLLRRLLLRxLLLxxRRRxLRRRRLRRRRLRRRRxLLLx',
             'xLLLxRLLLRRLLLRRLLLRxLLLxxRRRxRRRRLRRRRLRRRRLxLLLx',
             'xLLLxRLLLRRLLLRRLLLRxRRRxxLLLxLRRRLLRRRLLRRRLxRRRx',
             'xLLLxRLLLRRLLLRRLLLRxRRRxxLLLxLRRRRLRRRRLRRRRxLLLx',
             'xLLLxRLLLRRLLLRRLLLRxRRRxxLLLxRRRRLRRRRLRRRRLxLLLx',
             'xLLLxRLLLRRLLLRRLLLRxRRRxxRRRxLRRRLLRRRLLRRRLxLLLx',
             'xRRRxLLLLLLLLLLLLLLLxLLLxxLLLxRRRRRRRRRRRRRRRxRRRx',
             'xRRRxLLLLLLLLLLLLLLLxLLLxxRRRxLRRRRLRRRRLRRRRxRRRx',
             'xRRRxLLLLLLLLLLLLLLLxLLLxxRRRxRRRRLRRRRLRRRRLxRRRx',
             'xRRRxLLLLLLLLLLLLLLLxLLLxxRRRxRRRRRRRRRRRRRRRxLLLx',
             'xRRRxLLLLLLLLLLLLLLLxRRRxxLLLxLRRRRLRRRRLRRRRxRRRx',
             'xRRRxLLLLLLLLLLLLLLLxRRRxxLLLxRRRRLRRRRLRRRRLxRRRx',
             'xRRRxLLLLLLLLLLLLLLLxRRRxxLLLxRRRRRRRRRRRRRRRxLLLx',
             'xRRRxLLLLLLLLLLLLLLLxRRRxxRRRxLRRRLLRRRLLRRRLxRRRx',
             'xRRRxLLLLLLLLLLLLLLLxRRRxxRRRxLRRRRLRRRRLRRRRxLLLx',
             'xRRRxLLLLLLLLLLLLLLLxRRRxxRRRxRRRRLRRRRLRRRRLxLLLx',
             'xRRRxLLLLRLLLLRLLLLRxLLLxxLLLxLRRRRLRRRRLRRRRxRRRx',
             'xRRRxLLLLRLLLLRLLLLRxLLLxxLLLxRRRRLRRRRLRRRRLxRRRx',
             'xRRRxLLLLRLLLLRLLLLRxLLLxxLLLxRRRRRRRRRRRRRRRxLLLx',
             'xRRRxLLLLRLLLLRLLLLRxLLLxxRRRxLRRRLLRRRLLRRRLxRRRx',
             'xRRRxLLLLRLLLLRLLLLRxLLLxxRRRxLRRRRLRRRRLRRRRxLLLx',
             'xRRRxLLLLRLLLLRLLLLRxLLLxxRRRxRRRRLRRRRLRRRRLxLLLx',
             'xRRRxLLLLRLLLLRLLLLRxRRRxxLLLxLRRRLLRRRLLRRRLxRRRx',
             'xRRRxLLLLRLLLLRLLLLRxRRRxxLLLxLRRRRLRRRRLRRRRxLLLx',
             'xRRRxLLLLRLLLLRLLLLRxRRRxxLLLxRRRRLRRRRLRRRRLxLLLx',
             'xRRRxLLLLRLLLLRLLLLRxRRRxxRRRxLRRRLLRRRLLRRRLxLLLx',
             'xRRRxRLLLLRLLLLRLLLLxLLLxxLLLxLRRRRLRRRRLRRRRxRRRx',
             'xRRRxRLLLLRLLLLRLLLLxLLLxxLLLxRRRRLRRRRLRRRRLxRRRx',
             'xRRRxRLLLLRLLLLRLLLLxLLLxxLLLxRRRRRRRRRRRRRRRxLLLx',
             'xRRRxRLLLLRLLLLRLLLLxLLLxxRRRxLRRRLLRRRLLRRRLxRRRx',
             'xRRRxRLLLLRLLLLRLLLLxLLLxxRRRxLRRRRLRRRRLRRRRxLLLx',
             'xRRRxRLLLLRLLLLRLLLLxLLLxxRRRxRRRRLRRRRLRRRRLxLLLx',
             'xRRRxRLLLLRLLLLRLLLLxRRRxxLLLxLRRRLLRRRLLRRRLxRRRx',
             'xRRRxRLLLLRLLLLRLLLLxRRRxxLLLxLRRRRLRRRRLRRRRxLLLx',
             'xRRRxRLLLLRLLLLRLLLLxRRRxxLLLxRRRRLRRRRLRRRRLxLLLx',
             'xRRRxRLLLLRLLLLRLLLLxRRRxxRRRxLRRRLLRRRLLRRRLxLLLx',
             'xRRRxRLLLRRLLLRRLLLRxLLLxxLLLxLRRRLLRRRLLRRRLxRRRx',
             'xRRRxRLLLRRLLLRRLLLRxLLLxxLLLxLRRRRLRRRRLRRRRxLLLx',
             'xRRRxRLLLRRLLLRRLLLRxLLLxxLLLxRRRRLRRRRLRRRRLxLLLx',
             'xRRRxRLLLRRLLLRRLLLRxLLLxxRRRxLRRRLLRRRLLRRRLxLLLx',
             'xRRRxRLLLRRLLLRRLLLRxRRRxxLLLxLRRRLLRRRLLRRRLxLLLx'),
            moves_7x7x7,

            ("3Rw", "3Rw'", "3Lw", "3Lw'", "3Fw", "3Fw'", "3Bw", "3Bw'", "3Uw", "3Uw'", "3Dw", "3Dw'", # do not mess up staged centers
             "Rw", "Rw'", "Lw", "Lw'", "Fw", "Fw'", "Bw", "Bw'", "Uw", "Uw'", "Dw", "Dw'",             # do not mess up staged centers
             "3Rw2", "3Lw2", "3Fw2", "3Bw2", "Rw2", "Lw2", "Fw2", "Bw2"),                              # do not mess up solved UD

            # prune tables
            (parent.lt_LR_solve_inner_x_center_t_center_middle_oblique_edge,
             parent.lt_LR_solve_oblique_edge),
            linecount=6871798)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Left
            'x', parent_state[59], parent_state[60], parent_state[61], 'x',
            parent_state[65], parent_state[66], parent_state[67], parent_state[68], parent_state[69],
            parent_state[72], parent_state[73], parent_state[74], parent_state[75], parent_state[76],
            parent_state[79], parent_state[80], parent_state[81], parent_state[82], parent_state[83],
            'x', parent_state[87], parent_state[88], parent_state[89], 'x',

            # Right
            'x', parent_state[157], parent_state[158], parent_state[159], 'x',
            parent_state[163], parent_state[164], parent_state[165], parent_state[166], parent_state[167],
            parent_state[170], parent_state[171], parent_state[172], parent_state[173], parent_state[174],
            parent_state[177], parent_state[178], parent_state[179], parent_state[180], parent_state[181],
            'x', parent_state[185], parent_state[186], parent_state[187], 'x'
        ]

        result = ['x' if x in ('U', 'D', 'F', 'B') else x for x in result]
        result = ''.join(result)

        return result


class LookupTable777FBSolveInnerXCenterTCenterMiddleObliqueEdge(LookupTable):
    """
    lookup-table-7x7x7-step71-FB-inner-x-center-t-center-and-middle-oblique-edges.txt
    =================================================================================
    1 steps has 210 entries (0 percent, 0.00x previous step)
    2 steps has 770 entries (0 percent, 3.67x previous step)
    3 steps has 1,540 entries (0 percent, 2.00x previous step)
    4 steps has 5,950 entries (1 percent, 3.86x previous step)
    5 steps has 15,400 entries (4 percent, 2.59x previous step)
    6 steps has 44,310 entries (12 percent, 2.88x previous step)
    7 steps has 82,600 entries (24 percent, 1.86x previous step)
    8 steps has 120,960 entries (35 percent, 1.46x previous step)
    9 steps has 62,160 entries (18 percent, 0.51x previous step)
    10 steps has 8,820 entries (2 percent, 0.14x previous step)
    11 steps has 280 entries (0 percent, 0.03x previous step)

    Total: 343,000 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step71-FB-inner-x-center-t-center-and-middle-oblique-edges.txt',
            ('xxFxxxFFFxFFFFFxFFFxxxFxxxxBxxxBBBxBBBBBxBBBxxxBxx',
             'xxFxxxFFFxFFFFFxFFFxxxBxxxxFxxxBBBxBBBBBxBBBxxxBxx',
             'xxFxxxFFFxFFFFFxFFFxxxBxxxxBxxxBBBxFBBBBxBBBxxxBxx',
             'xxFxxxFFFxFFFFFxFFFxxxBxxxxBxxxBBBxBBBBFxBBBxxxBxx',
             'xxFxxxFFFxFFFFFxFFFxxxBxxxxBxxxBBBxBBBBBxBBBxxxFxx',
             'xxFxxxFFFxFFFFBxFFFxxxFxxxxFxxxBBBxBBBBBxBBBxxxBxx',
             'xxFxxxFFFxFFFFBxFFFxxxFxxxxBxxxBBBxFBBBBxBBBxxxBxx',
             'xxFxxxFFFxFFFFBxFFFxxxFxxxxBxxxBBBxBBBBFxBBBxxxBxx',
             'xxFxxxFFFxFFFFBxFFFxxxFxxxxBxxxBBBxBBBBBxBBBxxxFxx',
             'xxFxxxFFFxFFFFBxFFFxxxBxxxxFxxxBBBxFBBBBxBBBxxxBxx',
             'xxFxxxFFFxFFFFBxFFFxxxBxxxxFxxxBBBxBBBBFxBBBxxxBxx',
             'xxFxxxFFFxFFFFBxFFFxxxBxxxxFxxxBBBxBBBBBxBBBxxxFxx',
             'xxFxxxFFFxFFFFBxFFFxxxBxxxxBxxxBBBxFBBBFxBBBxxxBxx',
             'xxFxxxFFFxFFFFBxFFFxxxBxxxxBxxxBBBxFBBBBxBBBxxxFxx',
             'xxFxxxFFFxFFFFBxFFFxxxBxxxxBxxxBBBxBBBBFxBBBxxxFxx',
             'xxFxxxFFFxBFFFFxFFFxxxFxxxxFxxxBBBxBBBBBxBBBxxxBxx',
             'xxFxxxFFFxBFFFFxFFFxxxFxxxxBxxxBBBxFBBBBxBBBxxxBxx',
             'xxFxxxFFFxBFFFFxFFFxxxFxxxxBxxxBBBxBBBBFxBBBxxxBxx',
             'xxFxxxFFFxBFFFFxFFFxxxFxxxxBxxxBBBxBBBBBxBBBxxxFxx',
             'xxFxxxFFFxBFFFFxFFFxxxBxxxxFxxxBBBxFBBBBxBBBxxxBxx',
             'xxFxxxFFFxBFFFFxFFFxxxBxxxxFxxxBBBxBBBBFxBBBxxxBxx',
             'xxFxxxFFFxBFFFFxFFFxxxBxxxxFxxxBBBxBBBBBxBBBxxxFxx',
             'xxFxxxFFFxBFFFFxFFFxxxBxxxxBxxxBBBxFBBBFxBBBxxxBxx',
             'xxFxxxFFFxBFFFFxFFFxxxBxxxxBxxxBBBxFBBBBxBBBxxxFxx',
             'xxFxxxFFFxBFFFFxFFFxxxBxxxxBxxxBBBxBBBBFxBBBxxxFxx',
             'xxFxxxFFFxBFFFBxFFFxxxFxxxxFxxxBBBxFBBBBxBBBxxxBxx',
             'xxFxxxFFFxBFFFBxFFFxxxFxxxxFxxxBBBxBBBBFxBBBxxxBxx',
             'xxFxxxFFFxBFFFBxFFFxxxFxxxxFxxxBBBxBBBBBxBBBxxxFxx',
             'xxFxxxFFFxBFFFBxFFFxxxFxxxxBxxxBBBxFBBBFxBBBxxxBxx',
             'xxFxxxFFFxBFFFBxFFFxxxFxxxxBxxxBBBxFBBBBxBBBxxxFxx',
             'xxFxxxFFFxBFFFBxFFFxxxFxxxxBxxxBBBxBBBBFxBBBxxxFxx',
             'xxFxxxFFFxBFFFBxFFFxxxBxxxxFxxxBBBxFBBBFxBBBxxxBxx',
             'xxFxxxFFFxBFFFBxFFFxxxBxxxxFxxxBBBxFBBBBxBBBxxxFxx',
             'xxFxxxFFFxBFFFBxFFFxxxBxxxxFxxxBBBxBBBBFxBBBxxxFxx',
             'xxFxxxFFFxBFFFBxFFFxxxBxxxxBxxxBBBxFBBBFxBBBxxxFxx',
             'xxBxxxFFFxFFFFFxFFFxxxFxxxxFxxxBBBxBBBBBxBBBxxxBxx',
             'xxBxxxFFFxFFFFFxFFFxxxFxxxxBxxxBBBxFBBBBxBBBxxxBxx',
             'xxBxxxFFFxFFFFFxFFFxxxFxxxxBxxxBBBxBBBBFxBBBxxxBxx',
             'xxBxxxFFFxFFFFFxFFFxxxFxxxxBxxxBBBxBBBBBxBBBxxxFxx',
             'xxBxxxFFFxFFFFFxFFFxxxBxxxxFxxxBBBxFBBBBxBBBxxxBxx',
             'xxBxxxFFFxFFFFFxFFFxxxBxxxxFxxxBBBxBBBBFxBBBxxxBxx',
             'xxBxxxFFFxFFFFFxFFFxxxBxxxxFxxxBBBxBBBBBxBBBxxxFxx',
             'xxBxxxFFFxFFFFFxFFFxxxBxxxxBxxxBBBxFBBBFxBBBxxxBxx',
             'xxBxxxFFFxFFFFFxFFFxxxBxxxxBxxxBBBxFBBBBxBBBxxxFxx',
             'xxBxxxFFFxFFFFFxFFFxxxBxxxxBxxxBBBxBBBBFxBBBxxxFxx',
             'xxBxxxFFFxFFFFBxFFFxxxFxxxxFxxxBBBxFBBBBxBBBxxxBxx',
             'xxBxxxFFFxFFFFBxFFFxxxFxxxxFxxxBBBxBBBBFxBBBxxxBxx',
             'xxBxxxFFFxFFFFBxFFFxxxFxxxxFxxxBBBxBBBBBxBBBxxxFxx',
             'xxBxxxFFFxFFFFBxFFFxxxFxxxxBxxxBBBxFBBBFxBBBxxxBxx',
             'xxBxxxFFFxFFFFBxFFFxxxFxxxxBxxxBBBxFBBBBxBBBxxxFxx',
             'xxBxxxFFFxFFFFBxFFFxxxFxxxxBxxxBBBxBBBBFxBBBxxxFxx',
             'xxBxxxFFFxFFFFBxFFFxxxBxxxxFxxxBBBxFBBBFxBBBxxxBxx',
             'xxBxxxFFFxFFFFBxFFFxxxBxxxxFxxxBBBxFBBBBxBBBxxxFxx',
             'xxBxxxFFFxFFFFBxFFFxxxBxxxxFxxxBBBxBBBBFxBBBxxxFxx',
             'xxBxxxFFFxFFFFBxFFFxxxBxxxxBxxxBBBxFBBBFxBBBxxxFxx',
             'xxBxxxFFFxBFFFFxFFFxxxFxxxxFxxxBBBxFBBBBxBBBxxxBxx',
             'xxBxxxFFFxBFFFFxFFFxxxFxxxxFxxxBBBxBBBBFxBBBxxxBxx',
             'xxBxxxFFFxBFFFFxFFFxxxFxxxxFxxxBBBxBBBBBxBBBxxxFxx',
             'xxBxxxFFFxBFFFFxFFFxxxFxxxxBxxxBBBxFBBBFxBBBxxxBxx',
             'xxBxxxFFFxBFFFFxFFFxxxFxxxxBxxxBBBxFBBBBxBBBxxxFxx',
             'xxBxxxFFFxBFFFFxFFFxxxFxxxxBxxxBBBxBBBBFxBBBxxxFxx',
             'xxBxxxFFFxBFFFFxFFFxxxBxxxxFxxxBBBxFBBBFxBBBxxxBxx',
             'xxBxxxFFFxBFFFFxFFFxxxBxxxxFxxxBBBxFBBBBxBBBxxxFxx',
             'xxBxxxFFFxBFFFFxFFFxxxBxxxxFxxxBBBxBBBBFxBBBxxxFxx',
             'xxBxxxFFFxBFFFFxFFFxxxBxxxxBxxxBBBxFBBBFxBBBxxxFxx',
             'xxBxxxFFFxBFFFBxFFFxxxFxxxxFxxxBBBxFBBBFxBBBxxxBxx',
             'xxBxxxFFFxBFFFBxFFFxxxFxxxxFxxxBBBxFBBBBxBBBxxxFxx',
             'xxBxxxFFFxBFFFBxFFFxxxFxxxxFxxxBBBxBBBBFxBBBxxxFxx',
             'xxBxxxFFFxBFFFBxFFFxxxFxxxxBxxxBBBxFBBBFxBBBxxxFxx',
             'xxBxxxFFFxBFFFBxFFFxxxBxxxxFxxxBBBxFBBBFxBBBxxxFxx'),
            linecount=343000)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Front
            'xx', parent_state[109], 'xx',
            'x', parent_state[115], parent_state[116], parent_state[117], 'x',
            parent_state[121], parent_state[122], parent_state[123], parent_state[124], parent_state[125],
            'x', parent_state[129], parent_state[130], parent_state[131], 'x',
            'xx', parent_state[137], 'xx',

            # Back
            'xx', parent_state[207], 'xx',
            'x', parent_state[213], parent_state[214], parent_state[215], 'x',
            parent_state[219], parent_state[220], parent_state[221], parent_state[222], parent_state[223],
            'x', parent_state[227], parent_state[228], parent_state[229], 'x',
            'xx', parent_state[235], 'xx'
        ]

        result = ['x' if x in ('U', 'D', 'L', 'R') else x for x in result]
        result = ''.join(result)

        return result


class LookupTable777FBSolveObliqueEdge(LookupTable):
    """
    lookup-table-7x7x7-step72-FB-oblique-edges.txt
    ==============================================
    1 steps has 182 entries (0 percent, 0.00x previous step)
    2 steps has 616 entries (0 percent, 3.38x previous step)
    3 steps has 2,044 entries (0 percent, 3.32x previous step)
    4 steps has 8,576 entries (2 percent, 4.20x previous step)
    5 steps has 21,516 entries (6 percent, 2.51x previous step)
    6 steps has 60,392 entries (17 percent, 2.81x previous step)
    7 steps has 105,050 entries (30 percent, 1.74x previous step)
    8 steps has 106,464 entries (31 percent, 1.01x previous step)
    9 steps has 35,772 entries (10 percent, 0.34x previous step)
    10 steps has 2,368 entries (0 percent, 0.07x previous step)
    11 steps has 20 entries (0 percent, 0.01x previous step)

    Total: 343,000 entries
    """

    def __init__(self, parent):
        LookupTable.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step72-FB-oblique-edges.txt',
            ('xFFFxFxxxFFxxxFFxxxFxFFFxxBBBxBxxxBBxxxBBxxxBxBBBx',
             'xFFFxFxxxFFxxxFFxxxFxBBBxxFFFxBxxxBBxxxBBxxxBxBBBx',
             'xFFFxFxxxFFxxxFFxxxFxBBBxxBBBxFxxxBFxxxBFxxxBxBBBx',
             'xFFFxFxxxFFxxxFFxxxFxBBBxxBBBxBxxxFBxxxFBxxxFxBBBx',
             'xFFFxFxxxFFxxxFFxxxFxBBBxxBBBxBxxxBBxxxBBxxxBxFFFx',
             'xFFFxFxxxBFxxxBFxxxBxFFFxxFFFxBxxxBBxxxBBxxxBxBBBx',
             'xFFFxFxxxBFxxxBFxxxBxFFFxxBBBxFxxxBFxxxBFxxxBxBBBx',
             'xFFFxFxxxBFxxxBFxxxBxFFFxxBBBxBxxxFBxxxFBxxxFxBBBx',
             'xFFFxFxxxBFxxxBFxxxBxFFFxxBBBxBxxxBBxxxBBxxxBxFFFx',
             'xFFFxFxxxBFxxxBFxxxBxBBBxxFFFxFxxxBFxxxBFxxxBxBBBx',
             'xFFFxFxxxBFxxxBFxxxBxBBBxxFFFxBxxxFBxxxFBxxxFxBBBx',
             'xFFFxFxxxBFxxxBFxxxBxBBBxxFFFxBxxxBBxxxBBxxxBxFFFx',
             'xFFFxFxxxBFxxxBFxxxBxBBBxxBBBxFxxxFFxxxFFxxxFxBBBx',
             'xFFFxFxxxBFxxxBFxxxBxBBBxxBBBxFxxxBFxxxBFxxxBxFFFx',
             'xFFFxFxxxBFxxxBFxxxBxBBBxxBBBxBxxxFBxxxFBxxxFxFFFx',
             'xFFFxBxxxFBxxxFBxxxFxFFFxxFFFxBxxxBBxxxBBxxxBxBBBx',
             'xFFFxBxxxFBxxxFBxxxFxFFFxxBBBxFxxxBFxxxBFxxxBxBBBx',
             'xFFFxBxxxFBxxxFBxxxFxFFFxxBBBxBxxxFBxxxFBxxxFxBBBx',
             'xFFFxBxxxFBxxxFBxxxFxFFFxxBBBxBxxxBBxxxBBxxxBxFFFx',
             'xFFFxBxxxFBxxxFBxxxFxBBBxxFFFxFxxxBFxxxBFxxxBxBBBx',
             'xFFFxBxxxFBxxxFBxxxFxBBBxxFFFxBxxxFBxxxFBxxxFxBBBx',
             'xFFFxBxxxFBxxxFBxxxFxBBBxxFFFxBxxxBBxxxBBxxxBxFFFx',
             'xFFFxBxxxFBxxxFBxxxFxBBBxxBBBxFxxxFFxxxFFxxxFxBBBx',
             'xFFFxBxxxFBxxxFBxxxFxBBBxxBBBxFxxxBFxxxBFxxxBxFFFx',
             'xFFFxBxxxFBxxxFBxxxFxBBBxxBBBxBxxxFBxxxFBxxxFxFFFx',
             'xFFFxBxxxBBxxxBBxxxBxFFFxxFFFxFxxxBFxxxBFxxxBxBBBx',
             'xFFFxBxxxBBxxxBBxxxBxFFFxxFFFxBxxxFBxxxFBxxxFxBBBx',
             'xFFFxBxxxBBxxxBBxxxBxFFFxxFFFxBxxxBBxxxBBxxxBxFFFx',
             'xFFFxBxxxBBxxxBBxxxBxFFFxxBBBxFxxxFFxxxFFxxxFxBBBx',
             'xFFFxBxxxBBxxxBBxxxBxFFFxxBBBxFxxxBFxxxBFxxxBxFFFx',
             'xFFFxBxxxBBxxxBBxxxBxFFFxxBBBxBxxxFBxxxFBxxxFxFFFx',
             'xFFFxBxxxBBxxxBBxxxBxBBBxxFFFxFxxxFFxxxFFxxxFxBBBx',
             'xFFFxBxxxBBxxxBBxxxBxBBBxxFFFxFxxxBFxxxBFxxxBxFFFx',
             'xFFFxBxxxBBxxxBBxxxBxBBBxxFFFxBxxxFBxxxFBxxxFxFFFx',
             'xFFFxBxxxBBxxxBBxxxBxBBBxxBBBxFxxxFFxxxFFxxxFxFFFx',
             'xBBBxFxxxFFxxxFFxxxFxFFFxxFFFxBxxxBBxxxBBxxxBxBBBx',
             'xBBBxFxxxFFxxxFFxxxFxFFFxxBBBxFxxxBFxxxBFxxxBxBBBx',
             'xBBBxFxxxFFxxxFFxxxFxFFFxxBBBxBxxxFBxxxFBxxxFxBBBx',
             'xBBBxFxxxFFxxxFFxxxFxFFFxxBBBxBxxxBBxxxBBxxxBxFFFx',
             'xBBBxFxxxFFxxxFFxxxFxBBBxxFFFxFxxxBFxxxBFxxxBxBBBx',
             'xBBBxFxxxFFxxxFFxxxFxBBBxxFFFxBxxxFBxxxFBxxxFxBBBx',
             'xBBBxFxxxFFxxxFFxxxFxBBBxxFFFxBxxxBBxxxBBxxxBxFFFx',
             'xBBBxFxxxFFxxxFFxxxFxBBBxxBBBxFxxxFFxxxFFxxxFxBBBx',
             'xBBBxFxxxFFxxxFFxxxFxBBBxxBBBxFxxxBFxxxBFxxxBxFFFx',
             'xBBBxFxxxFFxxxFFxxxFxBBBxxBBBxBxxxFBxxxFBxxxFxFFFx',
             'xBBBxFxxxBFxxxBFxxxBxFFFxxFFFxFxxxBFxxxBFxxxBxBBBx',
             'xBBBxFxxxBFxxxBFxxxBxFFFxxFFFxBxxxFBxxxFBxxxFxBBBx',
             'xBBBxFxxxBFxxxBFxxxBxFFFxxFFFxBxxxBBxxxBBxxxBxFFFx',
             'xBBBxFxxxBFxxxBFxxxBxFFFxxBBBxFxxxFFxxxFFxxxFxBBBx',
             'xBBBxFxxxBFxxxBFxxxBxFFFxxBBBxFxxxBFxxxBFxxxBxFFFx',
             'xBBBxFxxxBFxxxBFxxxBxFFFxxBBBxBxxxFBxxxFBxxxFxFFFx',
             'xBBBxFxxxBFxxxBFxxxBxBBBxxFFFxFxxxFFxxxFFxxxFxBBBx',
             'xBBBxFxxxBFxxxBFxxxBxBBBxxFFFxFxxxBFxxxBFxxxBxFFFx',
             'xBBBxFxxxBFxxxBFxxxBxBBBxxFFFxBxxxFBxxxFBxxxFxFFFx',
             'xBBBxFxxxBFxxxBFxxxBxBBBxxBBBxFxxxFFxxxFFxxxFxFFFx',
             'xBBBxBxxxFBxxxFBxxxFxFFFxxFFFxFxxxBFxxxBFxxxBxBBBx',
             'xBBBxBxxxFBxxxFBxxxFxFFFxxFFFxBxxxFBxxxFBxxxFxBBBx',
             'xBBBxBxxxFBxxxFBxxxFxFFFxxFFFxBxxxBBxxxBBxxxBxFFFx',
             'xBBBxBxxxFBxxxFBxxxFxFFFxxBBBxFxxxFFxxxFFxxxFxBBBx',
             'xBBBxBxxxFBxxxFBxxxFxFFFxxBBBxFxxxBFxxxBFxxxBxFFFx',
             'xBBBxBxxxFBxxxFBxxxFxFFFxxBBBxBxxxFBxxxFBxxxFxFFFx',
             'xBBBxBxxxFBxxxFBxxxFxBBBxxFFFxFxxxFFxxxFFxxxFxBBBx',
             'xBBBxBxxxFBxxxFBxxxFxBBBxxFFFxFxxxBFxxxBFxxxBxFFFx',
             'xBBBxBxxxFBxxxFBxxxFxBBBxxFFFxBxxxFBxxxFBxxxFxFFFx',
             'xBBBxBxxxFBxxxFBxxxFxBBBxxBBBxFxxxFFxxxFFxxxFxFFFx',
             'xBBBxBxxxBBxxxBBxxxBxFFFxxFFFxFxxxFFxxxFFxxxFxBBBx',
             'xBBBxBxxxBBxxxBBxxxBxFFFxxFFFxFxxxBFxxxBFxxxBxFFFx',
             'xBBBxBxxxBBxxxBBxxxBxFFFxxFFFxBxxxFBxxxFBxxxFxFFFx',
             'xBBBxBxxxBBxxxBBxxxBxFFFxxBBBxFxxxFFxxxFFxxxFxFFFx',
             'xBBBxBxxxBBxxxBBxxxBxBBBxxFFFxFxxxFFxxxFFxxxFxFFFx'),
            linecount=343000)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Front
            'x', parent_state[108], parent_state[109], parent_state[110], 'x',
            parent_state[114], 'xxx', parent_state[118],
            parent_state[121], 'xxx', parent_state[125],
            parent_state[128], 'xxx', parent_state[132],
            'x', parent_state[136], parent_state[137], parent_state[138], 'x',

            # Back
            'x', parent_state[206], parent_state[207], parent_state[208], 'x',
            parent_state[212], 'xxx', parent_state[216],
            parent_state[219], 'xxx', parent_state[223],
            parent_state[226], 'xxx', parent_state[230],
            'x', parent_state[234], parent_state[235], parent_state[236], 'x'
        ]

        result = ['x' if x in ('U', 'D', 'L', 'R') else x for x in result]
        result = ''.join(result)

        return result


class LookupTableIDA777FBSolveInnerCentersAndObliqueEdges(LookupTableIDA):
    """
    lookup-table-7x7x7-step70-FB-solve-inner-center-and-oblique-edges.txt
    =====================================================================
    1 steps has 210 entries (0 percent, 0.00x previous step)
    2 steps has 770 entries (0 percent, 3.67x previous step)
    3 steps has 2,828 entries (0 percent, 3.67x previous step)
    4 steps has 15,158 entries (0 percent, 5.36x previous step)
    5 steps has 63,008 entries (0 percent, 4.16x previous step)
    6 steps has 290,588 entries (4 percent, 4.61x previous step)
    7 steps has 1,232,594 entries (17 percent, 4.24x previous step)
    8 steps has 5,266,642 entries (76 percent, 4.27x previous step)

    Total: 6,871,798 entries
    """

    def __init__(self, parent):
        LookupTableIDA.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step70-FB-solve-inner-center-and-oblique-edges.txt',
            ('xFFFxFFFFFFFFFFFFFFFxFFFxxBBBxBBBBBBBBBBBBBBBxBBBx',
             'xFFFxFFFFFFFFFFFFFFFxBBBxxFFFxBBBBBBBBBBBBBBBxBBBx',
             'xFFFxFFFFFFFFFFFFFFFxBBBxxBBBxFBBBBFBBBBFBBBBxBBBx',
             'xFFFxFFFFFFFFFFFFFFFxBBBxxBBBxBBBBFBBBBFBBBBFxBBBx',
             'xFFFxFFFFFFFFFFFFFFFxBBBxxBBBxBBBBBBBBBBBBBBBxFFFx',
             'xFFFxFFFFBFFFFBFFFFBxFFFxxFFFxBBBBBBBBBBBBBBBxBBBx',
             'xFFFxFFFFBFFFFBFFFFBxFFFxxBBBxFBBBBFBBBBFBBBBxBBBx',
             'xFFFxFFFFBFFFFBFFFFBxFFFxxBBBxBBBBFBBBBFBBBBFxBBBx',
             'xFFFxFFFFBFFFFBFFFFBxFFFxxBBBxBBBBBBBBBBBBBBBxFFFx',
             'xFFFxFFFFBFFFFBFFFFBxBBBxxFFFxFBBBBFBBBBFBBBBxBBBx',
             'xFFFxFFFFBFFFFBFFFFBxBBBxxFFFxBBBBFBBBBFBBBBFxBBBx',
             'xFFFxFFFFBFFFFBFFFFBxBBBxxFFFxBBBBBBBBBBBBBBBxFFFx',
             'xFFFxFFFFBFFFFBFFFFBxBBBxxBBBxFBBBFFBBBFFBBBFxBBBx',
             'xFFFxFFFFBFFFFBFFFFBxBBBxxBBBxFBBBBFBBBBFBBBBxFFFx',
             'xFFFxFFFFBFFFFBFFFFBxBBBxxBBBxBBBBFBBBBFBBBBFxFFFx',
             'xFFFxBFFFFBFFFFBFFFFxFFFxxFFFxBBBBBBBBBBBBBBBxBBBx',
             'xFFFxBFFFFBFFFFBFFFFxFFFxxBBBxFBBBBFBBBBFBBBBxBBBx',
             'xFFFxBFFFFBFFFFBFFFFxFFFxxBBBxBBBBFBBBBFBBBBFxBBBx',
             'xFFFxBFFFFBFFFFBFFFFxFFFxxBBBxBBBBBBBBBBBBBBBxFFFx',
             'xFFFxBFFFFBFFFFBFFFFxBBBxxFFFxFBBBBFBBBBFBBBBxBBBx',
             'xFFFxBFFFFBFFFFBFFFFxBBBxxFFFxBBBBFBBBBFBBBBFxBBBx',
             'xFFFxBFFFFBFFFFBFFFFxBBBxxFFFxBBBBBBBBBBBBBBBxFFFx',
             'xFFFxBFFFFBFFFFBFFFFxBBBxxBBBxFBBBFFBBBFFBBBFxBBBx',
             'xFFFxBFFFFBFFFFBFFFFxBBBxxBBBxFBBBBFBBBBFBBBBxFFFx',
             'xFFFxBFFFFBFFFFBFFFFxBBBxxBBBxBBBBFBBBBFBBBBFxFFFx',
             'xFFFxBFFFBBFFFBBFFFBxFFFxxFFFxFBBBBFBBBBFBBBBxBBBx',
             'xFFFxBFFFBBFFFBBFFFBxFFFxxFFFxBBBBFBBBBFBBBBFxBBBx',
             'xFFFxBFFFBBFFFBBFFFBxFFFxxFFFxBBBBBBBBBBBBBBBxFFFx',
             'xFFFxBFFFBBFFFBBFFFBxFFFxxBBBxFBBBFFBBBFFBBBFxBBBx',
             'xFFFxBFFFBBFFFBBFFFBxFFFxxBBBxFBBBBFBBBBFBBBBxFFFx',
             'xFFFxBFFFBBFFFBBFFFBxFFFxxBBBxBBBBFBBBBFBBBBFxFFFx',
             'xFFFxBFFFBBFFFBBFFFBxBBBxxFFFxFBBBFFBBBFFBBBFxBBBx',
             'xFFFxBFFFBBFFFBBFFFBxBBBxxFFFxFBBBBFBBBBFBBBBxFFFx',
             'xFFFxBFFFBBFFFBBFFFBxBBBxxFFFxBBBBFBBBBFBBBBFxFFFx',
             'xFFFxBFFFBBFFFBBFFFBxBBBxxBBBxFBBBFFBBBFFBBBFxFFFx',
             'xBBBxFFFFFFFFFFFFFFFxFFFxxFFFxBBBBBBBBBBBBBBBxBBBx',
             'xBBBxFFFFFFFFFFFFFFFxFFFxxBBBxFBBBBFBBBBFBBBBxBBBx',
             'xBBBxFFFFFFFFFFFFFFFxFFFxxBBBxBBBBFBBBBFBBBBFxBBBx',
             'xBBBxFFFFFFFFFFFFFFFxFFFxxBBBxBBBBBBBBBBBBBBBxFFFx',
             'xBBBxFFFFFFFFFFFFFFFxBBBxxFFFxFBBBBFBBBBFBBBBxBBBx',
             'xBBBxFFFFFFFFFFFFFFFxBBBxxFFFxBBBBFBBBBFBBBBFxBBBx',
             'xBBBxFFFFFFFFFFFFFFFxBBBxxFFFxBBBBBBBBBBBBBBBxFFFx',
             'xBBBxFFFFFFFFFFFFFFFxBBBxxBBBxFBBBFFBBBFFBBBFxBBBx',
             'xBBBxFFFFFFFFFFFFFFFxBBBxxBBBxFBBBBFBBBBFBBBBxFFFx',
             'xBBBxFFFFFFFFFFFFFFFxBBBxxBBBxBBBBFBBBBFBBBBFxFFFx',
             'xBBBxFFFFBFFFFBFFFFBxFFFxxFFFxFBBBBFBBBBFBBBBxBBBx',
             'xBBBxFFFFBFFFFBFFFFBxFFFxxFFFxBBBBFBBBBFBBBBFxBBBx',
             'xBBBxFFFFBFFFFBFFFFBxFFFxxFFFxBBBBBBBBBBBBBBBxFFFx',
             'xBBBxFFFFBFFFFBFFFFBxFFFxxBBBxFBBBFFBBBFFBBBFxBBBx',
             'xBBBxFFFFBFFFFBFFFFBxFFFxxBBBxFBBBBFBBBBFBBBBxFFFx',
             'xBBBxFFFFBFFFFBFFFFBxFFFxxBBBxBBBBFBBBBFBBBBFxFFFx',
             'xBBBxFFFFBFFFFBFFFFBxBBBxxFFFxFBBBFFBBBFFBBBFxBBBx',
             'xBBBxFFFFBFFFFBFFFFBxBBBxxFFFxFBBBBFBBBBFBBBBxFFFx',
             'xBBBxFFFFBFFFFBFFFFBxBBBxxFFFxBBBBFBBBBFBBBBFxFFFx',
             'xBBBxFFFFBFFFFBFFFFBxBBBxxBBBxFBBBFFBBBFFBBBFxFFFx',
             'xBBBxBFFFFBFFFFBFFFFxFFFxxFFFxFBBBBFBBBBFBBBBxBBBx',
             'xBBBxBFFFFBFFFFBFFFFxFFFxxFFFxBBBBFBBBBFBBBBFxBBBx',
             'xBBBxBFFFFBFFFFBFFFFxFFFxxFFFxBBBBBBBBBBBBBBBxFFFx',
             'xBBBxBFFFFBFFFFBFFFFxFFFxxBBBxFBBBFFBBBFFBBBFxBBBx',
             'xBBBxBFFFFBFFFFBFFFFxFFFxxBBBxFBBBBFBBBBFBBBBxFFFx',
             'xBBBxBFFFFBFFFFBFFFFxFFFxxBBBxBBBBFBBBBFBBBBFxFFFx',
             'xBBBxBFFFFBFFFFBFFFFxBBBxxFFFxFBBBFFBBBFFBBBFxBBBx',
             'xBBBxBFFFFBFFFFBFFFFxBBBxxFFFxFBBBBFBBBBFBBBBxFFFx',
             'xBBBxBFFFFBFFFFBFFFFxBBBxxFFFxBBBBFBBBBFBBBBFxFFFx',
             'xBBBxBFFFFBFFFFBFFFFxBBBxxBBBxFBBBFFBBBFFBBBFxFFFx',
             'xBBBxBFFFBBFFFBBFFFBxFFFxxFFFxFBBBFFBBBFFBBBFxBBBx',
             'xBBBxBFFFBBFFFBBFFFBxFFFxxFFFxFBBBBFBBBBFBBBBxFFFx',
             'xBBBxBFFFBBFFFBBFFFBxFFFxxFFFxBBBBFBBBBFBBBBFxFFFx',
             'xBBBxBFFFBBFFFBBFFFBxFFFxxBBBxFBBBFFBBBFFBBBFxFFFx',
             'xBBBxBFFFBBFFFBBFFFBxBBBxxFFFxFBBBFFBBBFFBBBFxFFFx'),
            moves_7x7x7,

            ("3Rw", "3Rw'", "3Lw", "3Lw'", "3Fw", "3Fw'", "3Bw", "3Bw'", "3Uw", "3Uw'", "3Dw", "3Dw'", # do not mess up staged centers
             "Rw", "Rw'", "Lw", "Lw'", "Fw", "Fw'", "Bw", "Bw'", "Uw", "Uw'", "Dw", "Dw'",             # do not mess up staged centers
             "3Rw2", "3Lw2", "3Fw2", "3Bw2", "Rw2", "Lw2", "Fw2", "Bw2",                               # do not mess up solved UD
             "L", "L'", "L2", "R", "R'", "R2"),                                                        # Do not mess up LRs reduced to 5x5x5 centers

            # prune tables
            (parent.lt_FB_solve_inner_x_center_t_center_middle_oblique_edge,
             parent.lt_FB_solve_oblique_edge),
            linecount=6871798)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Front
            'x', parent_state[108], parent_state[109], parent_state[110], 'x',
            parent_state[114], parent_state[115], parent_state[116], parent_state[117], parent_state[118],
            parent_state[121], parent_state[122], parent_state[123], parent_state[124], parent_state[125],
            parent_state[128], parent_state[129], parent_state[130], parent_state[131], parent_state[132],
            'x', parent_state[136], parent_state[137], parent_state[138], 'x',

            # Back
            'x', parent_state[206], parent_state[207], parent_state[208], 'x',
            parent_state[212], parent_state[213], parent_state[214], parent_state[215], parent_state[216],
            parent_state[219], parent_state[220], parent_state[221], parent_state[222], parent_state[223],
            parent_state[226], parent_state[227], parent_state[228], parent_state[229], parent_state[230],
            'x', parent_state[234], parent_state[235], parent_state[236], 'x'
        ]

        result = ['x' if x in ('U', 'D', 'L', 'R') else x for x in result]
        result = ''.join(result)

        return result


class LookupTableIDA777LFRBSolveInnerCentersAndObliqueEdges(LookupTableIDA):
    """
    lookup-table-7x7x7-step80-LFRB-solve-inner-center-and-oblique-edges.txt
    =======================================================================
    1 steps has 350 entries (0 percent, 0.00x previous step)
    2 steps has 2800 entries (0 percent, 8.00x previous step)
    3 steps has 18712 entries (1 percent, 6.68x previous step)
    4 steps has 131496 entries (11 percent, 7.03x previous step)
    5 steps has 966484 entries (86 percent, 7.35x previous step)

    Total: 1119842 entries
    """

    def __init__(self, parent):
        LookupTableIDA.__init__(
            self,
            parent,
            'lookup-table-7x7x7-step80-LFRB-solve-inner-center-and-oblique-edges.txt',
            lt_LFRB_solve_inner_centers_and_oblique_edges_state_targets, # There are 4900 of them
            moves_7x7x7,

            ("3Rw", "3Rw'", "3Lw", "3Lw'", "3Fw", "3Fw'", "3Bw", "3Bw'", "3Uw", "3Uw'", "3Dw", "3Dw'", # do not mess up staged centers
             "Rw", "Rw'", "Lw", "Lw'", "Fw", "Fw'", "Bw", "Bw'", "Uw", "Uw'", "Dw", "Dw'",             # do not mess up staged centers
             "3Rw2", "3Lw2", "3Fw2", "3Bw2", "Rw2", "Lw2", "Fw2", "Bw2",                               # do not mess up solved UD
             "L", "L'", "L2", "R", "R'", "R2"),                                                        # Do not mess up LRs reduced to 5x5x5 centers

            # prune tables
            (parent.lt_LR_solve_inner_x_center_t_center_middle_oblique_edge,
             parent.lt_LR_solve_oblique_edge,
             parent.lt_FB_solve_inner_x_center_t_center_middle_oblique_edge,
             parent.lt_FB_solve_oblique_edge),
            linecount=1119842)

    def state(self):
        parent_state = self.parent.state

        result = [
            # Left
            'x', parent_state[59], parent_state[60], parent_state[61], 'x',
            parent_state[65], parent_state[66], parent_state[67], parent_state[68], parent_state[69],
            parent_state[72], parent_state[73], parent_state[74], parent_state[75], parent_state[76],
            parent_state[79], parent_state[80], parent_state[81], parent_state[82], parent_state[83],
            'x', parent_state[87], parent_state[88], parent_state[89], 'x',

            # Front
            'x', parent_state[108], parent_state[109], parent_state[110], 'x',
            parent_state[114], parent_state[115], parent_state[116], parent_state[117], parent_state[118],
            parent_state[121], parent_state[122], parent_state[123], parent_state[124], parent_state[125],
            parent_state[128], parent_state[129], parent_state[130], parent_state[131], parent_state[132],
            'x', parent_state[136], parent_state[137], parent_state[138], 'x',

            # Right
            'x', parent_state[157], parent_state[158], parent_state[159], 'x',
            parent_state[163], parent_state[164], parent_state[165], parent_state[166], parent_state[167],
            parent_state[170], parent_state[171], parent_state[172], parent_state[173], parent_state[174],
            parent_state[177], parent_state[178], parent_state[179], parent_state[180], parent_state[181],
            'x', parent_state[185], parent_state[186], parent_state[187], 'x',

            # Back
            'x', parent_state[206], parent_state[207], parent_state[208], 'x',
            parent_state[212], parent_state[213], parent_state[214], parent_state[215], parent_state[216],
            parent_state[219], parent_state[220], parent_state[221], parent_state[222], parent_state[223],
            parent_state[226], parent_state[227], parent_state[228], parent_state[229], parent_state[230],
            'x', parent_state[234], parent_state[235], parent_state[236], 'x'
        ]

        result = ''.join(result)

        return result


class RubiksCube777(RubiksCubeNNNOddEdges):
    """
    For 7x7x7 centers
    - stage the UD inside 9 centers via 5x5x5
    - UD oblique edges
        - pair the two outside oblique edges via 6x6x6
        - build a lookup table to pair the middle oblique edges with the two
          outside oblique edges. The restriction being that if you do a 3Lw move
          you must also do a 3Rw' in order to keep the two outside oblique edges
          paired up...so it is a slice of the layer in the middle. This table
          should be (24!/(8!*16!))^2 or 540,917,591,800 so use IDA.
    - stage the rest of the UD centers via 5x5x5
    - stage the LR inside 9 centers via 5x5x5
    - LR oblique edges...use the same strategy as UD oblique edges
    - stage the rest of the LR centers via 5x5x5

    - solve the UD centers...this is (8!/(4!*4!))^6 or 117 billion so use IDA
    - solve the LR centers
    - solve the LR and FB centers

    For 7x7x7 edges
    - pair the middle 3 wings for each side via 5x5x5
    - pair the outer 2 wings with the paired middle 3 wings via 5x5x5


    Inheritance model
    -----------------
            RubiksCube
                |
        RubiksCubeNNNOddEdges
           /            \
    RubiksCubeNNNOdd RubiksCube777

    """

    def phase(self):
        if self._phase is None:
            self._phase = 'Stage UD centers'
            return self._phase

        if self._phase == 'Stage UD centers':
            if self.UD_centers_staged():
                self._phase = 'Stage LR centers'
            return self._phase

        if self._phase == 'Stage LR centers':
            if self.LR_centers_staged():
                self._phase = 'Solve Centers'

        if self._phase == 'Solve Centers':
            if self.centers_solved():
                self._phase = 'Pair Edges'

        if self._phase == 'Pair Edges':
            if not self.get_non_paired_edges():
                self._phase = 'Solve 3x3x3'

        return self._phase

    def sanity_check(self):
        edge_orbit_0 = (2, 6, 14, 42, 48, 44, 36, 8,
                        51, 55, 63, 91, 97, 93, 85, 57,
                        100, 104, 112, 140, 146, 142, 134, 106,
                        149, 153, 161, 189, 195, 191, 183, 155,
                        198, 202, 210, 238, 244, 240, 232, 204,
                        247, 251, 259, 287, 293, 289, 281, 253)

        edge_orbit_1 = (3, 5, 21, 35, 47, 45, 29, 15,
                        52, 54, 70, 84, 96, 94, 78, 64,
                        101, 103, 119, 133, 145, 143, 127, 113,
                        150, 152, 168, 182, 194, 192, 176, 162,
                        199, 201, 217, 231, 243, 241, 225, 211,
                        248, 250, 266, 280, 292, 290, 274, 260)

        edge_orbit_2 = (4, 28, 46, 22,
                        53, 77, 95, 71,
                        102, 126, 144, 120,
                        151, 175, 193, 169,
                        200, 224, 242, 218,
                        249, 273, 291, 267)

        corners = (1, 7, 43, 49,
                   50, 56, 92, 98,
                   99, 105, 141, 147,
                   148, 154, 190, 196,
                   197, 203, 239, 245,
                   246, 252, 288, 294)

        left_oblique_edge = (10, 20, 40, 30,
                             59, 69, 89, 79,
                             108, 118, 138, 128,
                             157, 167, 187, 177,
                             206, 216, 236, 226,
                             255, 265, 285, 275)

        right_oblique_edge = (12, 34, 38, 16,
                              61, 83, 87, 65,
                              110, 132, 136, 114,
                              159, 181, 185, 163,
                              208, 230, 234, 212,
                              257, 279, 283, 261)

        outside_x_centers = (9, 13, 37, 41,
                             58, 62, 86, 90,
                             107, 111, 135, 139,
                             156, 160, 184, 188,
                             205, 209, 233, 237,
                             254, 258, 282, 286)

        inside_x_centers = (17, 19, 31, 33,
                            66, 68, 80, 82,
                            115, 117, 129, 131,
                            164, 166, 178, 180,
                            213, 215, 227, 229,
                            262, 264, 276, 278)

        outside_t_centers = (11, 23, 27, 39,
                             60, 72, 76, 88,
                             109, 121, 125, 137,
                             158, 170, 174, 186,
                             207, 219, 223, 235,
                             256, 268, 272, 284)

        inside_t_centers = (18, 24, 26, 32,
                            67, 73, 75, 81,
                            116, 122, 124, 130,
                            165, 171, 173, 179,
                            214, 220, 222, 228,
                            263, 269, 271, 277)

        centers = (25, 74, 123, 172, 221, 270)

        self._sanity_check('edge-orbit-0', edge_orbit_0, 8)
        self._sanity_check('edge-orbit-1', edge_orbit_1, 8)
        self._sanity_check('edge-orbit-2', edge_orbit_2, 4)
        self._sanity_check('corners', corners, 4)
        self._sanity_check('left-oblique', left_oblique_edge, 4)
        self._sanity_check('right-oblique', right_oblique_edge, 4)
        self._sanity_check('outside x-centers', outside_x_centers, 4)
        self._sanity_check('inside x-centers', inside_x_centers, 4)
        self._sanity_check('outside t-centers', outside_t_centers, 4)
        self._sanity_check('inside t-centers', inside_t_centers, 4)
        self._sanity_check('centers', centers, 1)

    def lt_init(self):
        if self.lt_init_called:
            return
        self.lt_init_called = True

        self.lt_UD_oblique_edge_pairing_middle_only = LookupTable777UDObliqueEdgePairingMiddleOnly(self)
        self.lt_UD_oblique_edge_pairing_left_only = LookupTable777UDObliqueEdgePairingLeftOnly(self)
        self.lt_UD_oblique_edge_pairing_right_only = LookupTable777UDObliqueEdgePairingRightOnly(self)
        self.lt_UD_oblique_edge_pairing = LookupTableIDA777UDObliqueEdgePairing(self)

        self.lt_LR_oblique_edge_pairing_middle_only = LookupTable777LRObliqueEdgePairingMiddleOnly(self)
        self.lt_LR_oblique_edge_pairing_left_only = LookupTable777LRObliqueEdgePairingLeftOnly(self)
        self.lt_LR_oblique_edge_pairing_right_only = LookupTable777LRObliqueEdgePairingRightOnly(self)
        self.lt_LR_oblique_edge_pairing = LookupTableIDA777LRObliqueEdgePairing(self)

        self.lt_UD_solve_inner_centers_and_oblique_edges_center_only = LookupTable777UDSolveInnerCentersAndObliqueEdgesCenterOnly(self)
        self.lt_UD_solve_inner_centers_and_oblique_edges_edges_only = LookupTable777UDSolveInnerCentersAndObliqueEdgesEdgesOnly(self)
        self.lt_UD_solve_inner_centers_and_oblique_edges = LookupTableIDA777UDSolveInnerCentersAndObliqueEdges(self)

        self.lt_LR_solve_inner_x_center_t_center_middle_oblique_edge = LookupTable777LRSolveInnerXCenterTCenterMiddleObliqueEdge(self)
        self.lt_LR_solve_oblique_edge = LookupTable777LRSolveObliqueEdge(self)
        self.lt_LR_solve_inner_centers_and_oblique_edges = LookupTableIDA777LRSolveInnerCentersAndObliqueEdges(self)

        self.lt_FB_solve_inner_x_center_t_center_middle_oblique_edge = LookupTable777FBSolveInnerXCenterTCenterMiddleObliqueEdge(self)
        self.lt_FB_solve_oblique_edge = LookupTable777FBSolveObliqueEdge(self)
        self.lt_FB_solve_inner_centers_and_oblique_edges = LookupTableIDA777FBSolveInnerCentersAndObliqueEdges(self)

        self.lt_LFRB_solve_inner_centers_and_oblique_edges = LookupTableIDA777LFRBSolveInnerCentersAndObliqueEdges(self)

    def create_fake_555_from_inside_centers(self):

        # Create a fake 5x5x5 to stage the UD inner 5x5x5 centers
        fake_555 = RubiksCube555(solved_5x5x5, 'URFDLB')
        fake_555.cpu_mode = self.cpu_mode
        fake_555.lt_init()

        for x in range(1, 151):
            fake_555.state[x] = 'x'

        # Upper
        fake_555.state[7] = self.state[17]
        fake_555.state[8] = self.state[18]
        fake_555.state[9] = self.state[19]
        fake_555.state[12] = self.state[24]
        fake_555.state[13] = self.state[25]
        fake_555.state[14] = self.state[26]
        fake_555.state[17] = self.state[31]
        fake_555.state[18] = self.state[32]
        fake_555.state[19] = self.state[33]

        # Left
        fake_555.state[32] = self.state[66]
        fake_555.state[33] = self.state[67]
        fake_555.state[34] = self.state[68]
        fake_555.state[37] = self.state[73]
        fake_555.state[38] = self.state[74]
        fake_555.state[39] = self.state[75]
        fake_555.state[42] = self.state[80]
        fake_555.state[43] = self.state[81]
        fake_555.state[44] = self.state[82]

        # Front
        fake_555.state[57] = self.state[115]
        fake_555.state[58] = self.state[116]
        fake_555.state[59] = self.state[117]
        fake_555.state[62] = self.state[122]
        fake_555.state[63] = self.state[123]
        fake_555.state[64] = self.state[124]
        fake_555.state[67] = self.state[129]
        fake_555.state[68] = self.state[130]
        fake_555.state[69] = self.state[131]

        # Right
        fake_555.state[82] = self.state[164]
        fake_555.state[83] = self.state[165]
        fake_555.state[84] = self.state[166]
        fake_555.state[87] = self.state[171]
        fake_555.state[88] = self.state[172]
        fake_555.state[89] = self.state[173]
        fake_555.state[92] = self.state[178]
        fake_555.state[93] = self.state[179]
        fake_555.state[94] = self.state[180]

        # Back
        fake_555.state[107] = self.state[213]
        fake_555.state[108] = self.state[214]
        fake_555.state[109] = self.state[215]
        fake_555.state[112] = self.state[220]
        fake_555.state[113] = self.state[221]
        fake_555.state[114] = self.state[222]
        fake_555.state[117] = self.state[227]
        fake_555.state[118] = self.state[228]
        fake_555.state[119] = self.state[229]

        # Down
        fake_555.state[132] = self.state[262]
        fake_555.state[133] = self.state[263]
        fake_555.state[134] = self.state[264]
        fake_555.state[137] = self.state[269]
        fake_555.state[138] = self.state[270]
        fake_555.state[139] = self.state[271]
        fake_555.state[142] = self.state[276]
        fake_555.state[143] = self.state[277]
        fake_555.state[144] = self.state[278]
        fake_555.sanity_check()

        return fake_555

    def create_fake_555_from_outside_centers(self):

        # Create a fake 5x5x5 to solve 7x7x7 centers (they have been reduced to a 5x5x5)
        fake_555 = RubiksCube555(solved_5x5x5, 'URFDLB')
        fake_555.cpu_mode = self.cpu_mode
        fake_555.lt_init()

        for x in range(1, 151):
            fake_555.state[x] = 'x'

        # Upper
        fake_555.state[7] = self.state[9]
        fake_555.state[8] = self.state[11]
        fake_555.state[9] = self.state[13]
        fake_555.state[12] = self.state[23]
        fake_555.state[13] = self.state[25]
        fake_555.state[14] = self.state[27]
        fake_555.state[17] = self.state[37]
        fake_555.state[18] = self.state[39]
        fake_555.state[19] = self.state[41]

        # Left
        fake_555.state[32] = self.state[58]
        fake_555.state[33] = self.state[60]
        fake_555.state[34] = self.state[62]
        fake_555.state[37] = self.state[72]
        fake_555.state[38] = self.state[74]
        fake_555.state[39] = self.state[76]
        fake_555.state[42] = self.state[86]
        fake_555.state[43] = self.state[88]
        fake_555.state[44] = self.state[90]

        # Front
        fake_555.state[57] = self.state[107]
        fake_555.state[58] = self.state[109]
        fake_555.state[59] = self.state[111]
        fake_555.state[62] = self.state[121]
        fake_555.state[63] = self.state[123]
        fake_555.state[64] = self.state[125]
        fake_555.state[67] = self.state[135]
        fake_555.state[68] = self.state[137]
        fake_555.state[69] = self.state[139]

        # Right
        fake_555.state[82] = self.state[156]
        fake_555.state[83] = self.state[158]
        fake_555.state[84] = self.state[160]
        fake_555.state[87] = self.state[170]
        fake_555.state[88] = self.state[172]
        fake_555.state[89] = self.state[174]
        fake_555.state[92] = self.state[184]
        fake_555.state[93] = self.state[186]
        fake_555.state[94] = self.state[188]

        # Back
        fake_555.state[107] = self.state[205]
        fake_555.state[108] = self.state[207]
        fake_555.state[109] = self.state[209]
        fake_555.state[112] = self.state[219]
        fake_555.state[113] = self.state[221]
        fake_555.state[114] = self.state[223]
        fake_555.state[117] = self.state[233]
        fake_555.state[118] = self.state[235]
        fake_555.state[119] = self.state[237]

        # Down
        fake_555.state[132] = self.state[254]
        fake_555.state[133] = self.state[256]
        fake_555.state[134] = self.state[258]
        fake_555.state[137] = self.state[268]
        fake_555.state[138] = self.state[270]
        fake_555.state[139] = self.state[272]
        fake_555.state[142] = self.state[282]
        fake_555.state[143] = self.state[284]
        fake_555.state[144] = self.state[286]
        fake_555.sanity_check()

        return fake_555

    def group_inside_UD_centers(self):
        fake_555 = self.create_fake_555_from_inside_centers()
        fake_555.group_centers_stage_UD()

        for step in fake_555.solution:

            if step.startswith('5'):
                step = '7' + step[1:]
            elif step.startswith('3'):
                step = '4' + step[1:]
            elif 'w' in step:
                step = '3' + step

            self.rotate(step)

    def group_inside_LR_centers(self):
        fake_555 = self.create_fake_555_from_inside_centers()
        fake_555.lt_LR_centers_stage.solve()

        for step in fake_555.solution:

            if step.startswith('5'):
                step = '7' + step[1:]
            elif step.startswith('3'):
                raise Exception("5x5x5 solution has 3 wide turn")
            elif 'w' in step:
                step = '3' + step

            self.rotate(step)

    def solve_reduced_555_centers(self):
        fake_555 = self.create_fake_555_from_outside_centers()
        fake_555.group_centers()

        for step in fake_555.solution:

            if step == 'CENTERS_SOLVED':
                continue

            if step.startswith('5'):
                step = '7' + step[1:]

            elif step.startswith('3'):
                raise Exception("5x5x5 solution has 3 wide turn")

            self.rotate(step)

    def solve_reduced_555_t_centers(self):
        fake_555 = self.create_fake_555_from_outside_centers()
        fake_555.lt_ULFRBD_t_centers_solve.solve()

        for step in fake_555.solution:

            if step == 'CENTERS_SOLVED':
                continue

            if step.startswith('5'):
                step = '7' + step[1:]

            elif step.startswith('3'):
                raise Exception("5x5x5 solution has 3 wide turn")

            self.rotate(step)

    def create_fake_666_centers(self):

        # Create a fake 6x6x6 to stage the outside UD oblique edges
        fake_666 = RubiksCube666(solved_6x6x6, 'URFDLB')
        fake_666.cpu_mode = self.cpu_mode
        fake_666.lt_init()

        for x in range(1, 217):
            fake_666.state[x] = 'x'

        # Upper
        fake_666.state[8] = self.state[9]
        fake_666.state[9] = self.state[10]
        fake_666.state[10] = self.state[12]
        fake_666.state[11] = self.state[13]

        fake_666.state[14] = self.state[16]
        fake_666.state[15] = self.state[17]
        fake_666.state[16] = self.state[19]
        fake_666.state[17] = self.state[20]

        fake_666.state[20] = self.state[30]
        fake_666.state[21] = self.state[31]
        fake_666.state[22] = self.state[33]
        fake_666.state[23] = self.state[34]

        fake_666.state[26] = self.state[37]
        fake_666.state[27] = self.state[38]
        fake_666.state[28] = self.state[40]
        fake_666.state[29] = self.state[41]

        # Left
        fake_666.state[44] = self.state[58]
        fake_666.state[45] = self.state[59]
        fake_666.state[46] = self.state[61]
        fake_666.state[47] = self.state[62]
        fake_666.state[50] = self.state[65]
        fake_666.state[51] = self.state[66]
        fake_666.state[52] = self.state[68]
        fake_666.state[53] = self.state[69]
        fake_666.state[56] = self.state[79]
        fake_666.state[57] = self.state[80]
        fake_666.state[58] = self.state[82]
        fake_666.state[59] = self.state[83]
        fake_666.state[62] = self.state[86]
        fake_666.state[63] = self.state[87]
        fake_666.state[64] = self.state[89]
        fake_666.state[65] = self.state[90]

        # Front
        fake_666.state[80] = self.state[107]
        fake_666.state[81] = self.state[108]
        fake_666.state[82] = self.state[110]
        fake_666.state[83] = self.state[111]
        fake_666.state[86] = self.state[114]
        fake_666.state[87] = self.state[115]
        fake_666.state[88] = self.state[117]
        fake_666.state[89] = self.state[118]
        fake_666.state[92] = self.state[128]
        fake_666.state[93] = self.state[129]
        fake_666.state[94] = self.state[131]
        fake_666.state[95] = self.state[132]
        fake_666.state[98] = self.state[135]
        fake_666.state[99] = self.state[136]
        fake_666.state[100] = self.state[138]
        fake_666.state[101] = self.state[139]

        # Right
        fake_666.state[116] = self.state[156]
        fake_666.state[117] = self.state[157]
        fake_666.state[118] = self.state[159]
        fake_666.state[119] = self.state[160]
        fake_666.state[122] = self.state[163]
        fake_666.state[123] = self.state[164]
        fake_666.state[124] = self.state[166]
        fake_666.state[125] = self.state[167]
        fake_666.state[128] = self.state[177]
        fake_666.state[129] = self.state[178]
        fake_666.state[130] = self.state[180]
        fake_666.state[131] = self.state[181]
        fake_666.state[134] = self.state[184]
        fake_666.state[135] = self.state[185]
        fake_666.state[136] = self.state[187]
        fake_666.state[137] = self.state[188]

        # Back
        fake_666.state[152] = self.state[205]
        fake_666.state[153] = self.state[206]
        fake_666.state[154] = self.state[208]
        fake_666.state[155] = self.state[209]
        fake_666.state[158] = self.state[212]
        fake_666.state[159] = self.state[213]
        fake_666.state[160] = self.state[215]
        fake_666.state[161] = self.state[216]
        fake_666.state[164] = self.state[226]
        fake_666.state[165] = self.state[227]
        fake_666.state[166] = self.state[229]
        fake_666.state[167] = self.state[230]
        fake_666.state[170] = self.state[233]
        fake_666.state[171] = self.state[234]
        fake_666.state[172] = self.state[236]
        fake_666.state[173] = self.state[237]

        # Down
        fake_666.state[188] = self.state[254]
        fake_666.state[189] = self.state[255]
        fake_666.state[190] = self.state[257]
        fake_666.state[191] = self.state[258]
        fake_666.state[194] = self.state[261]
        fake_666.state[195] = self.state[262]
        fake_666.state[196] = self.state[264]
        fake_666.state[197] = self.state[265]
        fake_666.state[200] = self.state[275]
        fake_666.state[201] = self.state[276]
        fake_666.state[202] = self.state[278]
        fake_666.state[203] = self.state[279]
        fake_666.state[206] = self.state[282]
        fake_666.state[207] = self.state[283]
        fake_666.state[208] = self.state[285]
        fake_666.state[209] = self.state[286]
        fake_666.sanity_check()

        return fake_666

    def group_outside_UD_oblique_edges(self):
        fake_666 = self.create_fake_666_centers()
        fake_666.group_centers_stage_UD()

        for step in fake_666.solution:

            if step.startswith('6'):
                step = '7' + step[1:]

            self.rotate(step)

    def group_outside_LR_oblique_edges(self):
        fake_666 = self.create_fake_666_centers()
        fake_666.lt_LR_oblique_edge_pairing.solve()

        for step in fake_666.solution:

            if step.startswith('6'):
                step = '7' + step[1:]

            self.rotate(step)

    def group_centers_guts(self, oblique_edges_only=False):
        self.lt_init()

        # Uses 5x5x5 solver to stage the inner x-centers and inner t-centers for UD
        self.group_inside_UD_centers()
        log.info("Inside UD centers staged, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Uses 6x6x6 solver to pair the "outside" oblique edges for UD
        self.group_outside_UD_oblique_edges()
        log.info("Outside UD oblique edges paired, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Now pair the "inside" oblique edge with the already paired outside oblique edges...for UD
        self.lt_UD_oblique_edge_pairing.solve()
        log.info("UD oblique edges paired, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Uses 5x5x5 solver to stage the inner x-centers and inner t-centers for LR
        self.group_inside_LR_centers()
        log.info('post group_inside_LR_centers')
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Uses 6x6x6 solver to pair the "outside" oblique edges for LR
        self.group_outside_LR_oblique_edges()
        log.info('post group_outside_LR_oblique_edges')
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Now pair the "inside" oblique edge with the already paired outside oblique edges...for LR
        self.lt_LR_oblique_edge_pairing.solve()
        log.info("inner x-center and oblique edges staged, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Test pruning tables
        #self.lt_UD_solve_inner_centers_and_oblique_edges_center_only.solve()
        #self.lt_UD_solve_inner_centers_and_oblique_edges_edges_only.solve()
        #log.info("UD inner-centers solved, UD oblique edges paired, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        #self.print_cube()
        #sys.exit(0)

        self.lt_UD_solve_inner_centers_and_oblique_edges.solve()
        log.info("UD inner-centers solved, UD oblique edges paired, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")


        # Test prune tables
        #self.lt_LR_solve_inner_x_center_t_center_middle_oblique_edge.solve()
        #self.lt_LR_solve_oblique_edge.solve()
        #self.print_cube()
        #log.info("%d steps in" % self.get_solution_len_minus_rotates(self.solution))
        #sys.exit(0)

        # TODO if we build the LFRB table out to 6-deep can we avoid staging LR and FB?
        # Reduce LFRB centers to 5x5x5 via
        # Reduce LR centers to 5x5x5 by solving the inner x-centers, t-centers and oblique edges
        self.lt_LR_solve_inner_centers_and_oblique_edges.solve()
        log.info("LR inner-centers solved and oblique edges paired, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Test prune tables
        #self.lt_FB_solve_inner_x_center_t_center_middle_oblique_edge.solve()
        #self.lt_FB_solve_oblique_edge.solve()
        #self.print_cube()
        #log.info("%d steps in" % self.get_solution_len_minus_rotates(self.solution))
        #sys.exit(0)

        # Reduce FB centers to 5x5x5 by solving the inner x-centers, t-centers and oblique edges
        self.lt_FB_solve_inner_centers_and_oblique_edges.solve()
        log.info("FB inner-centers solved and oblique edges paired, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Reduce LRFB centers to 5x5x5 by solving the inner x-centers, t-centers and oblique edges
        self.lt_LFRB_solve_inner_centers_and_oblique_edges.solve()
        log.info("LFRB centers reduced to 5x5x5 centers, %d steps in" % self.get_solution_len_minus_rotates(self.solution))
        self.print_cube()
        log.info("")
        log.info("")
        log.info("")
        log.info("")

        # Centers are now reduced to 5x5x5 centers
        if oblique_edges_only:
            # If we are here it is because a larger cube is using the 7x7x7 solver to
            # solve their centers. We must "solve" the t-centers in this scenario.
            self.solve_reduced_555_t_centers()
            log.info("Took %d steps to solve oblique edges and t-centers" % self.get_solution_len_minus_rotates(self.solution))
            self.print_cube()
        else:
            self.solve_reduced_555_centers()
            log.info("Took %d steps to solve centers" % self.get_solution_len_minus_rotates(self.solution))
            self.print_cube()


# There are 4900 of them so I put them at the bottom of the file
lt_LFRB_solve_inner_centers_and_oblique_edges_state_targets = (
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xLLLxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLLLLLLLLLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxLLLLRLLLLRLLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRRRRRRRRRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRRRRRRRRRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLLRLLLLRLLLLxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxBBBxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFBBFFFBBFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxBFFFFBFFFFBFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFBFFFFBFFFFBxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxRRRxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRRLRRRRLRRRRxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxRRRRLRRRRLRRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxBBBxxRRRxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxRRRxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRRLRRRRLRRRRxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxRRRRLRRRRLRRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxLLLxxFFFxFFFFFFFFFFFFFFFxFFFxxRRRxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxBBBxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFBBFFFBBFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxBFFFFBFFFFBFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBFFBBBFFBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFBFFFFBFFFFBxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxFFFx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBFBBBBFBBBBFxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxFBBBBFBBBBFBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxBBBxxLLLxLRRRLLRRRLLRRRLxLLLxxFFFxBBBBBBBBBBBBBBBxBBBx',
    'xRRRxRLLLRRLLLRRLLLRxRRRxxFFFxFFFFFFFFFFFFFFFxFFFxxLLLxLRRRLLRRRLLRRRLxLLLxxBBBxBBBBBBBBBBBBBBBxBBBx',
)
